@@ -170,13 +170,16 @@ void app_normal_screen (Application *app) {
 static void callback_file_open (GtkButton * button, Application * app) {
     gchar *filename = NULL;
     filename = _app_file_chooser(app, GTK_FILE_CHOOSER_ACTION_OPEN);
-    if (filename)
+    if (filename) {
         app_open_location (filename, app);
+        g_free (filename);
+    }
 }
 
-static void callback_file_go (GtkButton * button, Application * app) {
+static void callback_file_go (GtkButton *button, Application *app) {
     const gchar *val = gtk_combo_box_get_active_text (
                 GTK_COMBO_BOX (app->location_bar));
+    (void)button;
     if (val && val[0]) {
         gtk_window_set_title (app->window, val);
         openFile (app, val);
@@ -197,6 +200,7 @@ static void callback_file_save_as (GtkButton * button, Application * app) {
             info = g_strdup_printf ("Saving as %s failed (%s)", filename, strerror (errno));
         hildon_banner_show_information (GTK_WIDGET (app->window), NULL, info);
         g_free (info);
+        g_free (filename);
     }
 }
 
@@ -230,6 +234,16 @@ locationbarLostFocus (GtkWidget *gw, GdkEventFocus *gef, Application *app) {
     return false;
 }
 
+gboolean locationbarKeyEvent (GtkWidget *w, GdkEventKey *e, Application *app) {
+    (void)w;
+    switch (e->keyval) {
+        case GDK_Return:
+            callback_file_go (NULL, app);
+            return true;
+    }
+    return false;
+}
+
 static bool first_time_exposed;
 
 static
@@ -530,6 +544,9 @@ int main (int argc, char* argv[]) {
     signal(SIGABRT, sig_abort);
     signal(SIGINT, sig_abort);
     signal(SIGSEGV, sig_abort);
+#ifdef KMPLAYER_DEBUG
+    signal(SIGUSR1, sig_usr1);
+#endif
 
     app = app_new ();
 
@@ -621,6 +638,8 @@ int main (int argc, char* argv[]) {
 
     g_signal_connect (G_OBJECT (gtk_bin_get_child (GTK_BIN(app->location_bar))),
             "focus-in-event", GTK_SIGNAL_FUNC (locationbarGainFocus), app);
+    g_signal_connect (G_OBJECT (gtk_bin_get_child (GTK_BIN(app->location_bar))),
+            "key-press-event", GTK_SIGNAL_FUNC (locationbarKeyEvent), app);
     g_signal_connect (G_OBJECT (app->playlist_view), "focus-in-event",
             GTK_SIGNAL_FUNC (locationbarLostFocus), app);
     g_signal_connect (G_OBJECT (app->infopanel_view), "focus-in-event",
