@@ -135,6 +135,11 @@ void openFile (Application *app, const gchar *path) MAEMO_NO_EXPORT;
 const char *saveFile (void *ctr, const gchar *path) MAEMO_NO_EXPORT;
 void newFile (Application *app) MAEMO_NO_EXPORT;
 
+/*#define KMPLAYER_DEBUG*/
+#ifdef KMPLAYER_DEBUG
+void sig_usr1 (int i);
+#endif
+
 #ifdef __cplusplus
 }
 #endif
