@@ -145,7 +145,7 @@ KDE_NO_EXPORT void Playlist::defer () {
         const char *tmp = (const char *)path;
         struct stat st;
         if (stat (tmp, &st))
-            tmp = "/usr/share/doc/kmplayer/playlist.xml";
+            tmp = "/usr/share/applications/kmplayer/playlist.xml";
         readFromFile (tmp);
     }
 }
