@@ -84,7 +84,7 @@ KDE_NO_EXPORT void RP::Imfl::activate () {
                 break;
         }
     if (duration > 0)
-        duration_timer = document ()->setTimeout (this, duration * 100);
+        duration_timer = document ()->postEvent (this, new TimerEvent (duration * 100));
     else if (!timings_count)
         finish ();
 }
@@ -93,7 +93,7 @@ KDE_NO_EXPORT void RP::Imfl::finish () {
     debugLog () << "RP::Imfl::finish " << endl;
     Mrl::finish ();
     if (duration_timer) {
-        document ()->cancelTimer (duration_timer);
+        document ()->cancelEvent (duration_timer);
         duration_timer = 0;
     }
     for (NodePtr n = firstChild (); n; n = n->nextSibling ())
@@ -129,15 +129,12 @@ KDE_NO_EXPORT void RP::Imfl::deactivate () {
     rp_surface = Mrl::getSurface (NULL);
 }
 
-KDE_NO_EXPORT bool RP::Imfl::handleEvent (EventPtr event) {
+KDE_NO_EXPORT bool RP::Imfl::handleEvent (Event *event) {
     if (event->id () == event_timer) {
-        TimerEvent * te = static_cast <TimerEvent *> (event.ptr ());
-        if (te->timer_info == duration_timer) {
-            //debugLog () << "RP::Imfl timer " << duration << endl;
-            duration_timer = 0;
-            if (unfinished ())
-                finish ();
-        }
+        //debugLog () << "RP::Imfl timer " << duration << endl;
+        duration_timer = 0;
+        if (unfinished ())
+            finish ();
     }
     return true;
 }
@@ -228,7 +225,7 @@ KDE_NO_EXPORT void RP::Image::deactivate () {
     }
 }
 
-bool RP::Image::handleEvent (EventPtr event) {
+bool RP::Image::handleEvent (Event *event) {
     if (event->id () != event_media_ready)
         return Mrl::handleEvent (event);
     dataArrived ();
@@ -239,8 +236,8 @@ KDE_NO_EXPORT void RP::Image::dataArrived () {
     //debugLog () << "RP::Image::remoteReady" << endl;
     ImageMedia *im = static_cast <ImageMedia *> (media_object);
     if (!im->isEmpty ()) {
-        width = im->cached_img->image->width ();
-        height = im->cached_img->image->height ();
+        width = im->cached_img->width;
+        height = im->cached_img->height;
     }
     postpone_lock = 0L;
 }
@@ -304,7 +301,7 @@ KDE_NO_EXPORT void RP::TimingsBase::activate () {
             srch = a->value ().toInt ();
         }
     }
-    start_timer = document ()->setTimeout (this, start *100);
+    start_timer = document ()->postEvent (this, new TimerEvent (start *100));
 }
 
 KDE_NO_EXPORT void RP::TimingsBase::deactivate () {
@@ -313,17 +310,17 @@ KDE_NO_EXPORT void RP::TimingsBase::deactivate () {
     setState (state_deactivated);
 }
 
-KDE_NO_EXPORT bool RP::TimingsBase::handleEvent (EventPtr event) {
+KDE_NO_EXPORT bool RP::TimingsBase::handleEvent (Event *event) {
     if (event->id () == event_timer) {
-        TimerEvent * te = static_cast <TimerEvent *> (event.ptr ());
-        if (te->timer_info == update_timer && duration > 0) {
+        TimerEvent * te = static_cast <TimerEvent *> (event);
+        if (event == update_timer && duration > 0) {
             update (200 * ++curr_step / duration);
             te->interval = true;
-        } else if (te->timer_info == start_timer) {
+        } else if (event == start_timer) {
             start_timer = 0;
-            duration_timer = document ()->setTimeout (this, duration * 100);
+            duration_timer = document()->postEvent (this, new TimerEvent(duration * 100));
             begin ();
-        } else if (te->timer_info == duration_timer) {
+        } else if (event == duration_timer) {
             duration_timer = 0;
             update (100);
             finish ();
@@ -331,7 +328,7 @@ KDE_NO_EXPORT bool RP::TimingsBase::handleEvent (EventPtr event) {
             return false;
         return true;
     } else if (event->id () == event_postponed) {
-        if (!static_cast <PostponedEvent *> (event.ptr ())->is_postponed) {
+        if (!static_cast <PostponedEvent *> (event)->is_postponed) {
             document_postponed = 0L; // disconnect
             update (duration > 0 ? 0 : 100);
         }
@@ -346,7 +343,7 @@ KDE_NO_EXPORT void RP::TimingsBase::begin () {
         target->begin ();
     if (duration > 0) {
         steps = duration / 2; // 5/s updates
-        update_timer = document ()->setTimeout (this, 200); // 200ms
+        update_timer = document ()->postEvent (this, new TimerEvent (200)); // 200ms
         curr_step = 1;
     }
 }
@@ -361,14 +358,14 @@ KDE_NO_EXPORT void RP::TimingsBase::update (int percentage) {
 KDE_NO_EXPORT void RP::TimingsBase::finish () {
     progress = 100;
     if (start_timer) {
-        document ()->cancelTimer (start_timer);
+        document ()->cancelEvent (start_timer);
         start_timer = 0;
     } else if (duration_timer) {
-        document ()->cancelTimer (duration_timer);
+        document ()->cancelEvent (duration_timer);
         duration_timer = 0;
     }
     if (update_timer) {
-        document ()->cancelTimer (update_timer);
+        document ()->cancelEvent (update_timer);
         update_timer = 0;
     }
     document_postponed = 0L; // disconnect
