@@ -33,16 +33,6 @@ class ImageMedia;
 
 typedef std::vector <String> StringList;
 
-/*
- * Event signaled before the actual starting takes place. Use by SMIL::Excl
- * to stop possible other children
- */
-class ToBeStartedEvent : public Event {
-public:
-    ToBeStartedEvent (NodePtr n);
-    NodePtrW node;
-};
-
 /*
  * Interpretation of sizes
  */
@@ -91,7 +81,8 @@ public:
 class KMPLAYER_NO_EXPORT Runtime {
 public:
     enum TimingState {
-        timings_reset = 0, timings_began, timings_started, timings_stopped
+        timings_reset = 0, timings_began,
+        timings_started, timings_paused, timings_stopped
     };
     enum DurationTime { begin_time = 0, duration_time, end_time, durtime_last };
     enum Duration {
@@ -124,20 +115,27 @@ public:
         int offset;
         ConnectionPtr connection;
     } durations [(const int) durtime_last];
-    void started ();
+    void setDuration ();
+    bool started () const {
+        return timingstate == timings_started || timingstate == timings_paused;
+    }
     void stopped ();
     KDE_NO_EXPORT DurationItem & beginTime () { return durations[begin_time]; }
     KDE_NO_EXPORT DurationItem & durTime () { return durations[duration_time]; }
     KDE_NO_EXPORT DurationItem & endTime () { return durations [end_time]; }
+
+    TimingState timingstate;
+    TimingState unpaused_state;
+    int repeat_count;
+    EventPtrW start_timer;
+    EventPtrW duration_timer;
+    EventPtrW started_timer;
+    EventPtrW stopped_timer;
+    NodePtrW paused_by;
+    unsigned int paused_time;
 private:
     void setDurationItem (DurationTime item, const String & val);
-public:
-    TimingState timingstate;
-    int repeat_count;
-protected:
     NodePtrW element;
-    TimerInfoPtrW start_timer;
-    TimerInfoPtrW duration_timer;
 };
 
 class KMPLAYER_NO_EXPORT MouseListeners {
@@ -182,6 +180,7 @@ const short id_node_animate = 133;
 const short id_node_title = 140;
 const short id_node_param = 141;
 const short id_node_meta = 142;
+const short id_node_priorityclass = 143;
 const short id_node_anchor = 150;
 const short id_node_area = 151;
 const short id_node_first = id_node_smil;
@@ -208,8 +207,8 @@ public:
     void closed ();
     void childDone (NodePtr child);
     bool expose () const;
-    bool handleEvent (EventPtr event);
-    void accept (Visitor *);
+    bool handleEvent (Event *event);
+    void accept (Visitor *v) { v->visit (this); }
     void jump (const String & id);
     static Smil * findSmilNode (Node * node);
 
@@ -243,7 +242,7 @@ public:
     void childDone (NodePtr child);
     void deactivate ();
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual bool handleEvent (EventPtr event);
+    virtual bool handleEvent (Event *event);
     /**
      * repaints region, calls scheduleRepaint(x,y,w,h) on view
      */
@@ -267,6 +266,7 @@ public:
     unsigned int background_color;
     String background_image;
     ShowBackground show_background;
+    Fit fit;
 protected:
     RegionBase (NodePtr & d, short id);
     PostponePtr postpone_lock;               // pause while loading bg image
@@ -400,19 +400,19 @@ public:
     void closed ();
     void activate ();
     void begin ();
+    void pause ();
     void finish ();
     void deactivate ();
     void reset ();
     bool expose () const { return false; }
     void childBegan (NodePtr child);
     void childDone (NodePtr child);
-    virtual bool handleEvent (EventPtr event);
+    virtual bool handleEvent (Event *event);
     virtual NodeRefListPtr listeners (unsigned int event_id);
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
     void init ();
     virtual void parseParam (const TrieString &, const String &);
     Runtime * runtime ();
-    static Runtime::DurationItem * getDuration (NodePtr n);
     static bool isTimedMrl (const Node *n);
     static bool keepContent (Node *n);
     static Fill getDefaultFill (NodePtr n);
@@ -455,7 +455,7 @@ public:
     void activate ();
     void begin ();
     void deactivate ();
-    bool handleEvent (EventPtr);
+    bool handleEvent (Event *);
     void setJumpNode (NodePtr);
 protected:
     KDE_NO_CDTOR_EXPORT GroupBase (NodePtr & d, short id) : TimedMrl (d, id) {}
@@ -505,13 +505,37 @@ class KMPLAYER_NO_EXPORT Excl : public GroupBase {
 public:
     KDE_NO_CDTOR_EXPORT Excl (NodePtr & d) : GroupBase (d, id_node_excl) {}
     KDE_NO_EXPORT const char * nodeName () const { return "excl"; }
+    NodePtr childFromTag (const String & tag);
     void begin ();
     void deactivate ();
     void childDone (NodePtr child);
-    virtual bool handleEvent (EventPtr event);
-private:
+    virtual bool handleEvent (Event *event);
+
     typedef ListNode <ConnectionPtr> ConnectionStoreItem;
     List <ConnectionStoreItem> started_event_list;
+    ConnectionPtr stopped_connection;
+    NodeRefList priority_queue;
+    NodePtrW cur_node;
+};
+
+/**
+ * A PriorityClass groups children within an Excl element
+ */
+class KMPLAYER_NO_EXPORT PriorityClass : public Element {
+public:
+    KDE_NO_CDTOR_EXPORT PriorityClass (NodePtr &d)
+        : Element (d, id_node_priorityclass) {}
+    KDE_NO_EXPORT const char * nodeName () const { return "priorityClass"; }
+    NodePtr childFromTag (const String & tag);
+    void init ();
+    void parseParam (const TrieString &, const String &);
+    void childDone (NodePtr child);
+    KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
+
+    enum { PeersStop, PeersPause, PeersDefer, PeersNever } peers;
+    enum { HigherStop, HigherPause } higher;
+    enum { LowerDefer, LowerNever } lower;
+    enum { PauseDisplayDisable, PauseDisplayHide, PauseDisplayShow } pause_display;
 };
 
 /*
@@ -596,7 +620,7 @@ public:
     SRect calculateBounds ();
     void boundsUpdate (); // recalculates and repaint old and new bounds
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual bool handleEvent (EventPtr event);
+    virtual bool handleEvent (Event *event);
     NodeRefListPtr listeners (unsigned int event_id);
 
     SurfacePtrW sub_surface;
@@ -612,8 +636,11 @@ public:
     unsigned int bitrate;
     unsigned int trans_step;
     unsigned int trans_steps;
+    EventPtrW trans_timer;
+    EventPtrW trans_out_timer;
     enum { sens_opaque, sens_transparent, sens_percentage } sensitivity;
     bool trans_out_active;
+    bool has_mouse;
 
 protected:
     virtual void clipStart ();
@@ -626,8 +653,6 @@ protected:
     ConnectionPtr region_mouse_leave;      // attached region has mouse left
     ConnectionPtr region_mouse_click;      // attached region is clicked
     ConnectionPtr region_attach;           // attached to region
-    TimerInfoPtrW trans_timer;
-    TimerInfoPtrW trans_out_timer;
     ConnectionPtr document_postponed;      // pause audio/video accordantly
     PostponePtr postpone_lock;
 };
@@ -655,7 +680,7 @@ public:
     virtual void begin ();
     virtual void accept (Visitor *);
     virtual void parseParam (const TrieString &, const String &);
-    virtual bool handleEvent (EventPtr event);
+    virtual bool handleEvent (Event *event);
     void dataArrived ();
 };
 
@@ -667,7 +692,7 @@ public:
     virtual void begin ();
     virtual void accept (Visitor *);
     virtual void parseParam (const TrieString &, const String &);
-    virtual bool handleEvent (EventPtr event);
+    virtual bool handleEvent (Event *event);
     void dataArrived ();
 
     String font_name;
@@ -722,14 +747,16 @@ public:
     virtual void finish ();
     virtual void deactivate ();
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual bool handleEvent (EventPtr event);
+    virtual bool handleEvent (Event *event);
+    virtual void accept (Visitor *v) { v->visit (this); }
     KDE_NO_EXPORT const char * nodeName () const { return "animate"; }
     PlayType playType () { return play_type_none; }
+
+    EventPtrW anim_timer;
 private:
     void applyStep ();
     bool timerTick();
 
-    TimerInfoPtrW anim_timer;
     enum { acc_none, acc_sum } accumulate;
     enum { add_replace, add_sum } additive;
     int change_by;
@@ -750,16 +777,18 @@ public:
     virtual void finish ();
     virtual void deactivate ();
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual bool handleEvent (EventPtr event);
+    virtual bool handleEvent (Event *event);
+    virtual void accept (Visitor *v) { v->visit (this); }
     KDE_NO_EXPORT const char * nodeName () const { return "animateMotion"; }
     PlayType playType () { return play_type_none; }
+
+    EventPtrW anim_timer;
 private:
     bool timerTick();
     bool checkTarget (Node *n);
     bool setInterval ();
     void applyStep ();
     bool getCoordinates (const String &coord, SizeType &x, SizeType &y);
-    TimerInfoPtrW anim_timer;
     enum { acc_none, acc_sum } accumulate;
     enum { add_replace, add_sum } additive;
     enum { calc_discrete, calc_linear, calc_paced, calc_spline } calcMode;
