@@ -66,6 +66,23 @@ static bool isPlayListMime (const String & mime) {
             !strcmp (mimestr, "application/x-mplayer2"));
 }
 
+#ifdef KMPLAYER_DEBUG
+static Application *debug_app;
+static Control *debug_control;
+
+void sig_usr1 (int) {
+    debugLog () << "Start dump" << endl;
+    debugLog () << debug_control->process () << " " <<
+        debug_control->blanking_timer << endl;
+    if (debug_control->process () && debug_control->process ()->mrl ())
+        debugLog () << debug_control->process ()->playing () <<
+            debug_control->process ()->mrl ()->src <<
+            debug_control->process ()->mrl ()->audio_only <<
+            debug_control->process ()->hasVideo () << endl;
+    debugLog () << "End dump" << endl;
+}
+#endif
+
 static bool isAudioOnlyMime (const String & mime) {
     const char * mimestr = (const char *) mime;
     return mimestr && (!strncmp ((const char *) mime, "audio/", 6));
@@ -86,8 +103,12 @@ Control::Control (Application * a)
       playlist_data (0L), m_config (new Config), m_app (a),
       paint_timer (0), update_tree_timer (0), blanking_timer (0),
       in_progress_update (false),
-      media_manager (new MediaManager (this))
-{}
+      media_manager (new MediaManager (this)) {
+#ifdef KMPLAYER_DEBUG
+    debug_app = a;
+    debug_control = this;
+#endif
+}
 
 Control::~Control () {
     delete m_process;
@@ -136,11 +157,13 @@ void Control::deinit () {
             rec->load_tree_version != rec->m_tree_version)
         rec->writeToFile (m_config->readEntry
                 (GCONF_KEY_RECENT_FILE, "/home/user/recent.xml"));
+    rec->dispose ();
     Playlist *playlist = static_cast<Playlist *>(playlists.ptr());
     if (playlist->resolved &&
             playlist->load_tree_version != playlist->m_tree_version)
         playlist->writeToFile (m_config->readEntry
                 (GCONF_KEY_PLAYLIST_FILE, "/home/user/playlist.xml"));
+    playlist->dispose ();
 }
 
 gboolean cb_blankingTimer (void * p) {
@@ -257,7 +280,7 @@ void Control::playCurrent () {
     if (playlist_data->document &&
             !playlist_data->document->active ()) {
         if (!playlist_data->current)
-            playlist_data->current = playlist_data->document;
+            setCurrent (playlist_data->document);
         else { // ugly code duplicate w/ back_request
             for (NodePtr p = playlist_data->current->parentNode(); p; p = p->parentNode())
                 p->setState (Element::state_activated);
@@ -295,6 +318,7 @@ void Control::playCurrent () {
             updateViewAreaSizes ();
             if (m_process->hasXv ())
                 XSync (GDK_DISPLAY_XDISPLAY (gtk_widget_get_display (VIEWAREA(m_app)->viewWidget ())), false);
+            VIEWAREA(m_app)->setEventFiltering (true);
             m_process->play (mrl);
         } else
             errorLog() << "playCurrent and mrl not resolved" << endl;
@@ -616,6 +640,14 @@ void Control::openUrl (const URL &, const String &, const String & /*srv*/) {
     errorMsg ("Open for target and mimetype is not supported");
 }
 
+bool Control::needsUpdate () const {
+    return viewArea ()->needsUpdateScreen ();
+}
+
+void Control::update () {
+    viewArea ()->updateScreen ();
+}
+
 bool Control::docTimerEvent () {
     int id = m_doc_timer;
     if (!playlist_data->document || !playlist_data->document->active ())
@@ -763,6 +795,7 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
     if (!p->mrl () && ns > Process::Ready) {
         p->stop (); // reschedule to the NotRunning state
     } else if (ns == Process::NotRunning) {
+        VIEWAREA(m_app)->setEventFiltering (false);
         if (p == m_process)
             if (playlist_data->document &&
                     playlist_data->document->state > Node::state_init) {
