@@ -94,6 +94,8 @@ public:
     void bitRates (int & prefered, int & maximal);
     void setTimeout (int ms);
     void openUrl (const URL &, const String &target, const String &srv);
+    bool needsUpdate () const;
+    void update ();
     void setProcess (Process * p) { m_process = p; }
     bool setProcess (const char *pname);
     bool setProcess (Mrl *mrl);
