@@ -112,60 +112,8 @@ KDE_NO_EXPORT void Connection::disconnect () {
 //-----------------------------------------------------------------------------
 
 KMPLAYER_NO_CDTOR_EXPORT
-TimerInfo::TimerInfo (NodePtr n, unsigned id, struct timeval & tv, int ms)
- : node (n), event_id (id), timeout (tv), milli_sec (ms) {}
-
-//-----------------------------------------------------------------------------
-
-Matrix::Matrix () : a (1.0), b (0.0), c (0.0), d (1.0), tx (0), ty (0) {}
-
-Matrix::Matrix (const Matrix & m)
- : a (m.a), b (m.b), c (m.c), d (m.d), tx (m.tx), ty (m.ty) {}
-
-Matrix::Matrix (Single xoff, Single yoff, float xscale, float yscale)
- : a (xscale), b (0.0), c (0.0), d (yscale), tx (xoff), ty (yoff) {}
-
-void Matrix::getXY (Single & x, Single & y) const {
-    x = Single (x * a) + tx;
-    y = Single (y * d) + ty;
-}
-
-void Matrix::getXYWH (Single & x, Single & y, Single & w, Single & h) const {
-    getXY (x, y);
-    w *= a;
-    h *= d;
-}
-
-void Matrix::invXYWH (Single & x, Single & y, Single & w, Single & h) const {
-    if (a > 0.00001 && d > 0.00001) {
-        w /= a;
-        h /= d;
-        x = Single ((x - tx) / a);
-        y = Single ((y - ty) / d);
-    } else {
-        warningLog () << "Not invering " << a << ", " << d << " scale" << endl;
-    }
-}
-
-void Matrix::transform (const Matrix & matrix) {
-    // TODO: rotate
-    a *= matrix.a;
-    d *= matrix.d;
-    tx = Single (tx * matrix.a) + matrix.tx;
-    ty = Single (ty * matrix.d) + matrix.ty;
-}
-
-void Matrix::scale (float sx, float sy) {
-    a *= sx;
-    d *= sy;
-    tx *= sx;
-    ty *= sy;
-}
-
-void Matrix::translate (Single x, Single y) {
-    tx += x;
-    ty += y;
-}
+TimerEvent::TimerEvent (int ms, unsigned eid)
+ : Event (NULL, event_timer), event_id (eid), milli_sec (ms), interval (false) {}
 
 //-----------------------------------------------------------------------------
 
@@ -448,9 +396,10 @@ NodeRefListPtr Node::listeners (unsigned int /*event_id*/) {
     return NodeRefListPtr ();
 }
 
-bool Node::handleEvent (EventPtr /*event*/) { return false; }
+bool Node::handleEvent (Event * /*event*/) { return false; }
 
-KDE_NO_EXPORT void Node::propagateEvent (EventPtr event) {
+KDE_NO_EXPORT void Node::propagateEvent (Event *event) {
+    EventPtr store (event);
     NodeRefListPtr nl = listeners (event->id ());
     if (nl)
         for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ())
@@ -776,7 +725,7 @@ Surface *Mrl::getSurface (Mrl *mrl) {
     return NULL;
 }
 
-bool Mrl::handleEvent (EventPtr) {
+bool Mrl::handleEvent (Event *) {
     return false;
 }
 
@@ -826,6 +775,11 @@ void Surface::markDirty () {
 
 //-----------------------------------------------------------------------------
 
+EventData::EventData (Node *t, Event *e, EventData *n)
+ : target (t), event (e), next (n) {}
+
+//-----------------------------------------------------------------------------
+
 Postpone::Postpone (NodePtr doc) : m_doc (doc) {
     if (m_doc)
         m_doc->document ()->timeOfDay (postponed_time);
@@ -845,7 +799,10 @@ Document::Document (const String & s, PlayListNotify * n)
    notify_listener (n),
    m_tree_version (0),
    m_PostponedListeners (new NodeRefList),
-   cur_timeout (-1), intimer (false) {
+   event_queue (NULL),
+   paused_queue (NULL),
+   cur_event (NULL),
+   cur_timeout (-1) {
     m_doc = m_self; // just-in-time setting fragile m_self to m_doc
     src = s;
     if (!s.isEmpty ())
@@ -857,15 +814,15 @@ Document::~Document () {
     //debugLog () << "~Document" << endl;
 }
 
-static NodePtr getElementByIdImpl (NodePtr n, const String & id, bool inter) {
+static Node *getElementByIdImpl (Node *n, const String & id, bool inter) {
     NodePtr elm;
     if (!n->isElementNode ())
-        return elm;
-    Element * e = convertNode <Element> (n);
+        return NULL;
+    Element *e = static_cast <Element *> (n);
     if (e->getAttribute (StringPool::attr_id) == id)
         return n;
-    for (NodePtr c = e->firstChild (); c; c = c->nextSibling ()) {
-        if (!inter && c->mrl () && c->mrl ()->opener == n)
+    for (Node *c = e->firstChild ().ptr (); c; c = c->nextSibling ().ptr ()) {
+        if (!inter && c->mrl () && c->mrl ()->opener.ptr () == n)
             continue;
         if ((elm = getElementByIdImpl (c, id, inter)))
             break;
@@ -873,11 +830,11 @@ static NodePtr getElementByIdImpl (NodePtr n, const String & id, bool inter) {
     return elm;
 }
 
-NodePtr Document::getElementById (const String & id) {
+Node *Document::getElementById (const String & id) {
     return getElementByIdImpl (this, id, true);
 }
 
-NodePtr Document::getElementById (NodePtr n, const String & id, bool inter) {
+Node *Document::getElementById (Node *n, const String & id, bool inter) {
     return getElementByIdImpl (n, id, inter);
 }
 
@@ -895,6 +852,8 @@ void Document::dispose () {
 
 void Document::activate () {
     first_event_time.tv_sec = 0;
+    last_paint_time.tv_sec = 0;
+    cur_timeout_time.tv_sec = 0;
     last_event_time = 0;
     Mrl::activate ();
 }
@@ -912,10 +871,15 @@ void Document::undefer () {
 
 void Document::reset () {
     Mrl::reset ();
-    if (timers.first ()) {
+    if (event_queue) {
         if (notify_listener)
             notify_listener->setTimeout (-1);
-        timers.clear ();
+        while (event_queue) {
+            EventData *ed = event_queue;
+            event_queue = ed->next;
+            delete ed;
+        }
+        cur_timeout = -1;
     }
     postpone_lock = 0L;
 }
@@ -931,6 +895,18 @@ static inline void addTime (struct timeval & tv, int ms) {
     tv.tv_usec = (tv.tv_usec + ms*1000) % 1000000;
 }
 
+/*static inline void subtractTime (struct timeval & tv, int ms) {
+    int sec = ms / 1000;
+    int msec = ms % 1000;
+    tv.tv_sec -= sec;
+    if (tv.tv_usec / 1000 >= msec) {
+        tv.tv_usec -= msec * 1000;
+    } else {
+        tv.tv_sec--;
+        tv.tv_usec = 1000000 - (msec - tv.tv_usec / 1000 );
+    }
+}*/
+
 void Document::timeOfDay (struct timeval & tv) {
     gettimeofday (&tv, 0L);
     if (!first_event_time.tv_sec) {
@@ -940,143 +916,248 @@ void Document::timeOfDay (struct timeval & tv) {
         last_event_time = diffTime (tv, first_event_time) / 100;
 }
 
-TimerInfoPtrW Document::setTimeout (NodePtr n, int ms, unsigned id) {
+static bool postponedSensible (Event *e) {
+    return e->id () == event_timer ||
+        e->id () == event_started ||
+        e->id () == event_stopped;
+}
+
+void Document::insertEvent (Node *n, Event *e, const struct timeval &tv) {
     if (!notify_listener)
-        return 0L;
-    TimerInfoPtr ti = timers.first ();
-    int pos = 0;
-    struct timeval tv;
-    timeOfDay (tv);
-    addTime (tv, ms);
-    for (; ti && diffTime (ti->timeout, tv) <= 0; ti = ti->nextSibling ()) {
-        pos++;
-        //debugLog () << "setTimeout tv:" << tv.tv_sec << "." << tv.tv_usec << " "  << ti->timeout.tv_sec << "." << ti->timeout.tv_usec << endl;
+        return;
+    bool postponed_sensible = postponedSensible (e);
+    EventData *prev = NULL;
+    EventData *ed = event_queue;
+    for (; ed; ed = ed->next) {
+        int diff = diffTime (ed->timeout, tv);
+        bool psens = postponedSensible (ed->event.ptr ());
+        if ((diff > 0 && postponed_sensible == psens) || (!postponed_sensible && psens))
+            break;
+        prev = ed;
     }
-    TimerInfo * tinfo = new TimerInfo (n, id, tv, ms);
-    timers.insertBefore (tinfo, ti);
+    ed = new EventData (n, e, ed);
+    ed->timeout = tv;
+    if (prev)
+        prev->next = ed;
+    else
+        event_queue = ed;
     //debugLog () << "setTimeout " << ms << " at:" << pos << " tv:" << tv.tv_sec << "." << tv.tv_usec << endl;
-    if (!postpone_ref && pos == 0 && !intimer) { // timer() does that too
-        cur_timeout = ms;
-        notify_listener->setTimeout (ms);
-    }
-    return tinfo;
 }
 
-void Document::cancelTimer (TimerInfoPtr tinfo) {
-    if (!postpone_ref && !intimer && tinfo == timers.first ()) {
-        //debugLog () << "cancel first" << endl;
-        TimerInfoPtr second = tinfo->nextSibling ();
-        if (second) {
-            struct timeval now;
-            timeOfDay (now);
-            int diff = diffTime (now, second->timeout);
-            cur_timeout = diff > 0 ? 0 : -diff;
-        } else
-            cur_timeout = -1;
-        notify_listener->setTimeout (cur_timeout);
-    }
-    timers.remove (tinfo);
-}
+void Document::setNextTimeout (const struct timeval &n) {
+    if (!cur_event) {              // if we're not processing events
+        int timeout = 0x7FFFFFFF;
+        static const int off = -1;
+        struct timeval now = n;
+        if (event_queue && active () &&
+                (!postpone_ref || !postponedSensible (event_queue->event.ptr ())))
+            timeout = diffTime (event_queue->timeout, now);
+        timeout = 0x7FFFFFFF != timeout ? (timeout > 0 ? timeout : 0) : off;
 
-bool Document::timer () {
-    struct timeval now = { 0, 0 }; // unset
-    int new_timeout = -1;
-    TimerInfoPtrW tinfo = timers.first (); // keep use_count on 1
-    NodePtrW guard = this;
-
-    intimer = true;
-    // handle max 100 timeouts with timeout set to now
-    for (int i = 0; !!tinfo && !postpone_ref && i < 100 && active (); ++i) {
-        if (tinfo && !tinfo->node) {
-            // some part of document has gone and didn't remove timer
-            errorLog () << "spurious timer" << endl;
-            for (; tinfo && !tinfo->node; tinfo = timers.first ())
-                timers.remove (tinfo);
-            tinfo = timers.first ();
-        }
-        if (!tinfo)
-            break;
-        TimerEvent * te = new TimerEvent (tinfo);
-        EventPtr e (te);
-        tinfo->node->handleEvent (e);
-        if (!guard)
-            return false;
-        if (tinfo) { // may be removed from timers and become 0
-            if (te->interval) {
-                TimerInfoPtr tinfo2 (tinfo); // prevent destruction
-                timers.remove (tinfo);
+        if (timeout && notify_listener->needsUpdate ()) {
+            int diff = diffTime (now, last_paint_time);
+            if (!last_paint_time.tv_sec ||
+                    diff > 35 || (timeout > 10 && diff > 25)) {
+                notify_listener->update ();
                 timeOfDay (now);
-                int drift = diffTime (now, tinfo2->timeout);
-                if (drift > tinfo2->milli_sec) {
-                    drift = tinfo2->milli_sec;
-                    //warningLog() << "time drift" << endl;
-                }
-                tinfo2->timeout = now;
-                addTime (tinfo2->timeout, tinfo2->milli_sec - drift);
-                TimerInfoPtr ti = timers.first ();
-                int pos = 0;
-                for (; ti && diffTime (tinfo2->timeout, ti->timeout) >= 0; ti = ti->nextSibling ()) {
-                    pos++;
-                }
-                timers.insertBefore (tinfo2, ti);
-            } else
-                timers.remove (tinfo);
+                last_paint_time = now;
+                if (notify_listener->needsUpdate () &&
+                        (off == timeout || timeout > 40))
+                    timeout = 40;
+            } else {
+                int next_repaint = 40 - diff;
+                if (off == timeout || next_repaint < timeout)
+                    timeout = next_repaint;
+            }
         }
-        tinfo = timers.first ();
-        if (!tinfo)
-            break;
-        if (!now.tv_sec)
-            timeOfDay (now);
-        int diff = diffTime (now, tinfo->timeout);
-        new_timeout = diff > 0 ? 0 : -diff;
-        if (new_timeout > 0)
-            break;
-    }
-    intimer = false;
 
-    // set new timeout to prevent interval timer events
-    if (notify_listener && !postpone_ref && tinfo && active ()) {
-        if (new_timeout != cur_timeout) {
-            cur_timeout = new_timeout;
-            notify_listener->setTimeout (cur_timeout);
+        int dtimer = cur_timeout - timeout;
+        addTime (now, timeout);
+        int dtimeout = diffTime (cur_timeout_time, now);
+        bool onoff = (off == timeout && cur_timeout != off) ||
+                (off == cur_timeout && timeout != off);
+        if (onoff || (timeout != off && (-3 > dtimeout || dtimeout > 3))) {
+            if (onoff || (timeout != off && (-3 > dtimer || dtimer > 3))) {
+                cur_timeout = timeout;
+                notify_listener->setTimeout (timeout);
+            }
+            cur_timeout_time = now;
         }
-        // else keep the timer, no new setTimeout
-    } else {
-        cur_timeout = -1;
-        notify_listener->setTimeout (-1); // kill timer
     }
-    return false;
+}
+
+void Document::updateTimeout () {
+    if (!cur_event && (-1 == cur_timeout || cur_timeout > 25)) {
+        struct timeval now;
+        timeOfDay (now);
+        setNextTimeout (now);
+    }
+}
+
+Event *Document::postEvent (Node *n, Event *e) {
+    int ms = e->id () == event_timer ? static_cast <TimerEvent *> (e)->milli_sec : 0;
+    struct timeval now, tv;
+    if (cur_event)
+        now = cur_event->timeout;
+    else
+        timeOfDay (now);
+    tv = now;
+    addTime (tv, ms);
+    insertEvent (n, e, tv);
+    if (postpone_ref || event_queue->event.ptr () == e)
+        setNextTimeout (now);
+    return e;
+}
+
+static EventData *findEvent (EventData *queue, EventData **prev, const Event *e) {
+    *prev = NULL;
+    for (EventData *ed = queue; ed; ed = ed->next) {
+        if (e == ed->event.ptr ())
+            return ed;
+        *prev = ed;
+    }
+    return NULL;
+}
+
+void Document::cancelEvent (Event *e) {
+    if (cur_event && cur_event->event.ptr () == e) {
+        cur_event->event = NULL;
+    } else {
+        EventData *prev;
+        EventData **queue = &event_queue;
+        EventData *ed = findEvent (event_queue, &prev, e);
+        if (!ed) {
+            ed = findEvent (paused_queue, &prev, e);
+            queue = &paused_queue;
+        }
+        if (ed) {
+            if (prev) {
+                prev->next = ed->next;
+            } else {
+                *queue = ed->next;
+                if (!cur_event && queue == &event_queue) {
+                    struct timeval now;
+                    if (event_queue) // save a sys call
+                        timeOfDay (now);
+                    setNextTimeout (now);
+                }
+            }
+            delete ed;
+        } else {
+            errorLog () << "Event not found" << endl;
+        }
+    }
+}
+
+void Document::pauseEvent (Event *e) {
+    if (cur_event && cur_event->event.ptr () == e) {
+        paused_queue = new EventData (cur_event->target, cur_event->event, paused_queue);
+        paused_queue->timeout = cur_event->timeout;
+    } else {
+        EventData *prev;
+        EventData *ed = findEvent (event_queue, &prev, e);
+        if (ed) {
+            if (prev)
+                prev->next = ed->next;
+            else
+                event_queue = ed->next;
+            ed->next = paused_queue;
+            paused_queue = ed;
+        } else {
+            errorLog () << "pauseEvent not found";
+        }
+    }
+}
+
+void Document::unpauseEvent (Event *e, int ms) {
+    EventData *prev;
+    EventData *ed = findEvent (paused_queue, &prev, e);
+    if (ed) {
+        if (prev)
+            prev->next = ed->next;
+        else
+            paused_queue = ed->next;
+        addTime (ed->timeout, ms);
+        insertEvent (ed->target, ed->event, ed->timeout);
+        delete ed;
+    } else {
+        errorLog () << "pauseEvent not found";
+    }
+}
+
+void Document::timer () {
+    struct timeval now;
+    timeOfDay (now);
+    cur_event = event_queue;
+    int diff = cur_event ? diffTime (cur_event->timeout, now) : 0;
+    if (cur_event && diff < 5) {
+        NodePtrW guard = this;
+        struct timeval start = cur_event->timeout;
+
+        // handle max 100 timeouts with timeout set to now
+        for (int i = 0; i < 100 && active (); ++i) {
+            if (postpone_ref && postponedSensible (cur_event->event.ptr ()))
+                break;
+            // remove from queue
+            event_queue = cur_event->next;
+
+            if (!cur_event->target) {
+                // some part of document has gone and didn't remove timer
+                errorLog () << "spurious timer" << endl;
+            } else {
+                EventData *ed = cur_event;
+                cur_event->target->handleEvent (cur_event->event.ptr ());
+                if (!guard) {
+                    delete ed;
+                    return;
+                }
+                if (cur_event->event && cur_event->event->id() == event_timer) {
+                    TimerEvent *te = static_cast <TimerEvent *> (cur_event->event.ptr ());
+                    if (te->interval) {
+                        te->interval = false; // reset interval
+                        addTime (cur_event->timeout, te->milli_sec);
+                        insertEvent (cur_event->target,
+                                cur_event->event.ptr (), cur_event->timeout);
+                    }
+                }
+            }
+            delete cur_event;
+            cur_event = event_queue;
+            if (!cur_event || diffTime (cur_event->timeout, start) > 5)
+                break;
+        }
+    }
+    cur_event = NULL;
+    setNextTimeout (now);
 }
 
 PostponePtr Document::postpone () {
     if (postpone_ref)
         return postpone_ref;
     debugLog () << "postponed " << endl;
-    if (!intimer && notify_listener) {
-        cur_timeout = -1;
-        notify_listener->setTimeout (-1);
-    }
     PostponePtr p = new Postpone (this);
     postpone_ref = p;
     propagateEvent (new PostponedEvent (true));
+    if (!cur_event) {
+        struct timeval now;
+        if (event_queue) // save a sys call
+            timeOfDay (now);
+        setNextTimeout (now);
+    }
     return p;
 }
 
-void Document::proceed (const struct timeval & postponed_time) {
+void Document::proceed (const struct timeval &postponed_time) {
     debugLog () << "proceed" << endl;
-    if (timers.first () && notify_listener) {
+    postpone_ref = NULL;
+    if (event_queue) {
         struct timeval now;
         timeOfDay (now);
         int diff = diffTime (now, postponed_time);
-        if (diff > 0) {
-            for (TimerInfoPtr t = timers.first (); t; t = t->nextSibling ())
-                addTime (t->timeout, diff);
-        }
-        if (!intimer) { // eg. postpone() + proceed() in same timer()
-            diff = diffTime (timers.first ()->timeout, now);
-            cur_timeout = diff < 0 ? 0 : diff;
-            notify_listener->setTimeout (cur_timeout);
-        }
+        for (EventData *ed = event_queue; ed; ed = ed->next)
+            if (ed->event && postponedSensible (ed->event))
+                addTime (ed->timeout, diff);
+        setNextTimeout (now);
     }
     propagateEvent (new PostponedEvent (false));
 }
