@@ -35,6 +35,7 @@ namespace KMPlayer {
 
 class Document;
 class Node;
+class Event;
 class Mrl;
 class Surface;
 class ElementPrivate;
@@ -206,6 +207,7 @@ ITEM_AS_POINTER(KMPlayer::Attribute)
  * Object should scale according the passed Fit value in SizedEvent
  */
 enum Fit {
+    fit_default,   // fill hidden
     fit_fill,      // fill complete area, no aspect preservation
     fit_hidden,    // keep aspect and don't scale, cut off what doesn't fit
     fit_meet,      // keep aspect and scale so that the smallest size just fits
@@ -213,27 +215,14 @@ enum Fit {
     fit_scroll     // keep aspect and don't scale, add scollbars if needed
 };
 
-/*
- * A generic event type
- */
-class KMPLAYER_EXPORT Event : public Item <Event> {
-public:
-    KDE_NO_CDTOR_EXPORT Event (unsigned int event_id) : m_event_id (event_id) {}
-    KDE_NO_CDTOR_EXPORT virtual ~Event () {}
-    KDE_NO_EXPORT unsigned int id () const { return m_event_id; }
-protected:
-    unsigned int m_event_id;
-};
-
-ITEM_AS_POINTER(KMPlayer::Event)
-
 extern const unsigned int event_pointer_clicked;
 extern const unsigned int event_pointer_moved;
 extern const unsigned int event_inbounds;
 extern const unsigned int event_outbounds;
-extern const unsigned int event_sized;
 extern const unsigned int event_postponed;
 extern const unsigned int event_timer;
+extern const unsigned int event_started;
+extern const unsigned int event_stopped;
 extern const unsigned int mediatype_attached;
 
 // convenient types
@@ -242,6 +231,7 @@ typedef Item<Node>::WeakType NodePtrW;
 typedef Item<Attribute>::SharedType AttributePtr;
 typedef Item<Attribute>::WeakType AttributePtrW;
 typedef Item<Event>::SharedType EventPtr;
+typedef Item<Event>::WeakType EventPtrW;
 typedef List<Node> NodeList;                 // eg. for Node's children
 typedef Item<NodeList>::SharedType NodeListPtr;
 typedef Item<NodeList>::WeakType NodeListPtrW;
@@ -346,11 +336,11 @@ public:
     /*
      * Event send to this node, return true if handled
      */
-    virtual bool handleEvent (EventPtr event);
+    virtual bool handleEvent (Event *event);
     /*
      * Dispatch Event to all listeners of event->id()
      */
-    void propagateEvent (EventPtr event);
+    void propagateEvent (Event *event);
     /**
      * Alternative to event handling is the Visitor pattern
      */
@@ -545,7 +535,7 @@ public:
      * By default support one event handler (eg. SMIL or RP child document)
      */
     virtual Surface *getSurface (Mrl *mrl);
-    virtual bool handleEvent (EventPtr event);
+    virtual bool handleEvent (Event *event);
 
     /**
      * If this Mrl is top node of external document, opener has the
@@ -602,6 +592,8 @@ public:
      * Request to open url with mimetype
      */
     virtual void openUrl (const URL &, const String &tgt, const String &srv)=0;
+    virtual bool needsUpdate () const = 0;
+    virtual void update () = 0;
 };
 
 class KMPLAYER_NO_EXPORT Surface : public TreeNode <Surface> {
@@ -630,32 +622,30 @@ protected:
 
 ITEM_AS_POINTER(KMPlayer::Surface)
 
-/**
- * To have a somewhat synchronized time base, node having timers should use
- * this. Idea is that if a node still waiting for network data, it can hold
- * this time line.
+/*
+ * A generic event type
  */
-class KMPLAYER_NO_EXPORT TimerInfo : public ListNodeBase <TimerInfo> {
+class KMPLAYER_EXPORT Event : public Item <Event> {
 public:
-    TimerInfo (NodePtr n, unsigned id, struct timeval & now, int ms);
-    KDE_NO_CDTOR_EXPORT ~TimerInfo () {}
-    NodePtrW node;
-    unsigned event_id;
-    struct timeval timeout;
-    int milli_sec;
+    KDE_NO_CDTOR_EXPORT Event (Node *n, unsigned int event_id)
+        : source (n), m_event_id (event_id) {}
+    KDE_NO_CDTOR_EXPORT virtual ~Event () {}
+    KDE_NO_EXPORT unsigned int id () const { return m_event_id; }
+    NodePtrW source;
+protected:
+    unsigned int m_event_id;
 };
 
-ITEM_AS_POINTER(KMPlayer::TimerInfo)
-typedef Item <TimerInfo>::SharedType TimerInfoPtr;
-typedef Item <TimerInfo>::WeakType TimerInfoPtrW;
+ITEM_AS_POINTER(KMPlayer::Event)
 
 /**
  * Event signaling a timer event
  */
 class KMPLAYER_NO_EXPORT TimerEvent : public Event {
 public:
-    TimerEvent (TimerInfoPtr tinfo);
-    TimerInfoPtrW timer_info;
+    TimerEvent (int ms, unsigned eid=0);
+    unsigned event_id;
+    int milli_sec;
     bool interval; // set to 'true' in 'Node::handleEvent()' to make it repeat
 };
 
@@ -684,6 +674,16 @@ public:
 typedef SharedPtr <Postpone> PostponePtr;
 typedef WeakPtr <Postpone> PostponePtrW;
 
+struct KMPLAYER_NO_EXPORT EventData {
+    EventData (Node *t, Event *e, EventData *n);
+
+    NodePtrW target;
+    EventPtr event;
+    struct timeval timeout;
+
+    EventData *next;
+};
+
 /**
  * The root of the DOM tree
  */
@@ -692,8 +692,8 @@ class KMPLAYER_EXPORT Document : public Mrl {
 public:
     Document (const String &, PlayListNotify * notify = 0L);
     ~Document ();
-    NodePtr getElementById (const String & id);
-    NodePtr getElementById (NodePtr start, const String & id, bool inter_doc);
+    Node *getElementById (const String & id);
+    Node *getElementById (Node *start, const String & id, bool inter_doc);
     /** All nodes have shared pointers to Document,
      * so explicitly dispose it (calls clear and set m_doc to 0L)
      * */
@@ -704,20 +704,20 @@ public:
     virtual void defer ();
     virtual void undefer ();
     virtual void reset ();
-    /**
-     * Ask for TimerEvent for Node n in ms milli-seconds.
-     * Returns weak ref to TimerInfo ptr, which is an item in the timers list
-     */
-    TimerInfoPtrW setTimeout (NodePtr n, int ms, unsigned id=0);
-    void cancelTimer (TimerInfoPtr ti);
+
+    Event *postEvent (Node *n, Event *event);
+    void cancelEvent (Event *event);
+    void pauseEvent (Event *e);
+    void unpauseEvent (Event *e, int ms);
+
     void timeOfDay (struct timeval &);
     PostponePtr postpone ();
     bool postponed () const { return !!postpone_ref; }
     /**
-     * Called by PlayListNotify, creates TimerEvent on first item in timers.
-     * Returns true if to repeat this same timeout FIXME.
+     * Called by PlayListNotify, processes events in event_queue with timeout set to now
      */
-    bool timer ();
+    void timer ();
+    void updateTimeout ();
     /**
      * Document has list of postponed listeners, eg. for running (gif)movies
      */
@@ -727,18 +727,24 @@ public:
      */
     virtual Surface *getSurface (Mrl *mrl);
 
-    List <TimerInfo> timers; //FIXME: make as connections
-    PlayListNotify * notify_listener;
+    PlayListNotify *notify_listener;
     unsigned int m_tree_version;
     unsigned int last_event_time;
 private:
     void proceed (const struct timeval & postponed_time);
+    void insertEvent (Node *n, Event *e, const struct timeval &tv);
+    void setNextTimeout (const struct timeval &now);
+
     PostponePtrW postpone_ref;
     PostponePtr postpone_lock;
     NodeRefListPtr m_PostponedListeners;
+    EventData *event_queue;
+    EventData *paused_queue;
+    EventData *cur_event;
     int cur_timeout;
     struct timeval first_event_time;
-    bool intimer;
+    struct timeval last_paint_time;
+    struct timeval cur_timeout_time;
 };
 
 /**
@@ -782,10 +788,13 @@ protected:
 };
 
 namespace SMIL {
+    class Smil;
     class RegionBase;
     class Region;
     class Layout;
     class Transition;
+    class Animate;
+    class AnimateMotion;
     class MediaType;
     class ImageMediaType;
     class TextMediaType;
@@ -793,6 +802,7 @@ namespace SMIL {
     class AVMediaType;
     class Brush;
     class TimedMrl;
+    class PriorityClass;
     class Anchor;
     class Area;
 }
@@ -812,10 +822,14 @@ public:
     KDE_NO_CDTOR_EXPORT Visitor () {}
     KDE_NO_CDTOR_EXPORT virtual ~Visitor () {}
     virtual void visit (Node *) {}
+    virtual void visit (SMIL::Smil *) {}
     virtual void visit (SMIL::Region *);
     virtual void visit (SMIL::Layout *);
     virtual void visit (SMIL::Transition *);
+    virtual void visit (SMIL::Animate *);
+    virtual void visit (SMIL::AnimateMotion *);
     virtual void visit (SMIL::TimedMrl *);
+    virtual void visit (SMIL::PriorityClass *);
     virtual void visit (SMIL::MediaType *);
     virtual void visit (SMIL::ImageMediaType *);
     virtual void visit (SMIL::TextMediaType *);
