@@ -5,25 +5,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <pango/pango-font.h>
-
-#define noname configure_xpm
-#include "configure.xpm"
-#undef noname
-#define noname folder_xpm
-#include "folder.xpm"
-#undef noname
-#define noname folder_grey_xpm
-#include "folder_grey.xpm"
-#undef noname
-#define noname playlist_xpm
-#include "playlist.xpm"
-#undef noname
-#define noname unknown_xpm
-#include "unknown.xpm"
-#undef noname
-#define noname video_xpm
-#include "video.xpm"
-#undef noname
+#include <hildon/hildon-defines.h>
 
 #include "kmplayerplaylist.h"
 #include "kmplayer.h"
@@ -36,7 +18,9 @@ static GdkPixbuf *folder_pix;
 static GdkPixbuf *auxiliary_pix;
 static GdkPixbuf *unknown_pix;
 static GdkPixbuf *config_pix;
+static GdkPixbuf *music_pix;
 static GdkPixbuf *video_pix;
+static GdkPixbuf *video_link_pix;
 static GdkPixbuf *list_pix;
 
 /*
@@ -95,8 +79,20 @@ enum {
     NUM_COLS
 };
 
-// gtk_tree_store_clear            (GtkTreeStore *tree_store);
-// 
+static GdkPixbuf *iconForNode (Node *n, int child_count) {
+    return n->isPlayable()
+        ? n->mrl ()->resolved
+            ? n->mrl ()->linkNode ()->audio_only
+                ? music_pix
+                : video_pix
+            : video_link_pix
+        : child_count > 0
+            ? n->auxiliaryNode ()
+                ? auxiliary_pix
+                : folder_pix
+            : unknown_pix;
+}
+
 static
 int populate (GtkTreeStore * tree_store, PlayListData * pld,
         NodePtr e, GtkTreeIter * riter, GtkTreeIter * piter,
@@ -116,7 +112,7 @@ int populate (GtkTreeStore * tree_store, PlayListData * pld,
         gtk_tree_store_append (tree_store, &iter, piter);
     else
         iter = *riter;
-    Mrl * mrl = e->mrl ();
+    Mrl *mrl = e->mrl ();
     String text (e->nodeName());
     if (mrl && !pld->show_all) {
         if (mrl->pretty_name.isEmpty ()) {
@@ -139,8 +135,8 @@ int populate (GtkTreeStore * tree_store, PlayListData * pld,
             COL_TEXT, (const char *) text, COL_ID, pld->id, -1);
     if (pld->current == e)
         *fpath = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), &iter);
-    if (e->active ())
-        ; //ensureItemVisible (item);
+    //if (e->active ())
+    //    ; //ensureItemVisible (item);
     if (e->isElementNode ()) {
         AttributePtr a = convertNode<Element> (e)->attributes ()->first ();
         if (a) {
@@ -168,11 +164,77 @@ int populate (GtkTreeStore * tree_store, PlayListData * pld,
     }
     for (NodePtr c = e->firstChild (); c; c = c->nextSibling ())
         child_count += populate (tree_store, pld, c, riter, &iter, fpath);
-    GdkPixbuf *icon = piter ? e->isPlayable() ? video_pix : (child_count > 0 ? (e->auxiliaryNode () ? auxiliary_pix : folder_pix) : unknown_pix) : pld->icon;
+    GdkPixbuf *icon = piter ? iconForNode (e.ptr (), child_count) : pld->icon;
     gtk_tree_store_set (tree_store, &iter, COL_ICON, icon, -1);
     return child_count + 1;
 }
 
+/*static void dumpIter (tkTreeModel *mode, GtkTreeIter *iter) {
+    GValue value = { 0 , };
+    gtk_tree_model_get_value (model, &iter, COL_TEXT, &value);
+    const gchar *val = g_value_get_string (&value);
+    debugLog () << "found iter: " << val << endl;
+    g_value_unset (&value);
+}*/
+
+static gboolean findNode (PlayListData *pld, GtkTreeModel *model, Node *node, GtkTreeIter *iter) {
+    Node *p = node->parentNode ();
+    if (p) {
+        if (!findNode (pld, model, p, iter))
+            return false;
+        bool first_exposed = false;
+        for (Node *n=p->firstChild().ptr(); n; n=n->nextSibling().ptr()) {
+            if (pld->show_all || n->expose ()) {
+                if (!first_exposed) {
+                    first_exposed = true;
+                    GtkTreeIter piter = *iter;
+                    if (!gtk_tree_model_iter_children (model, iter, &piter))
+                        return false;
+                    if (pld->show_all && n->isElementNode () &&
+                            convertNode<Element>(n)->attributes()->first() &&
+                            !gtk_tree_model_iter_next (model, iter))
+                        return false;
+                } else if (!gtk_tree_model_iter_next (model, iter)) {
+                    return false;
+                }
+            }
+            if (n == node)
+                return true;
+        }
+        errorLog() << "findNode node not found" << endl;
+        return false;
+    } else {
+        return true;
+    }
+}
+
+void KMPlayer::updatePlayTree (Application *app) {
+    app->in_tree_update = true;
+    PlayListData *pld = CONTROL(app)->playlist_data;
+
+    Node *node = pld->current.ptr ();
+    if (!pld->show_all)
+        while (node && !node->expose ())
+            node = node->parentNode ().ptr ();
+    if (!node) {
+        // undo selection
+        return;
+    }
+    GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (app->playlist_view));
+    GtkTreeIter iter;
+    if (!gtk_tree_model_get_iter_first (model, &iter ))
+        return;
+    if (findNode (pld, model, node, &iter)) {
+        //GdkPixbuf *icon = piter ? iconForNode (e.ptr (), 0) : pld->icon;
+        GdkPixbuf *icon = iconForNode (node, 0);
+        gtk_tree_store_set (GTK_TREE_STORE (model), &iter, COL_ICON, icon, -1);
+        gtk_tree_selection_select_iter (gtk_tree_view_get_selection (GTK_TREE_VIEW (app->playlist_view)), &iter);
+    } else {
+        errorLog() << "findNode return false" << endl;
+    }
+    app->in_tree_update = false;
+}
+
 void KMPlayer::updateTree (Application * app, short id) {
     app->in_tree_update = true;
     GtkTreeIter valid_iter, *iter;
@@ -478,12 +540,23 @@ GtkWidget * createTreeView (Application * app) {
     GtkWidget         *view;
     //GValue            prop_value = { TRUE, };
 
-    folder_pix = gdk_pixbuf_new_from_xpm_data ((const char**) folder_xpm);
-    auxiliary_pix = gdk_pixbuf_new_from_xpm_data((const char**)folder_grey_xpm);
-    unknown_pix = gdk_pixbuf_new_from_xpm_data ((const char**) unknown_xpm);
-    config_pix = gdk_pixbuf_new_from_xpm_data ((const char**) configure_xpm);
-    video_pix = gdk_pixbuf_new_from_xpm_data ((const char**) video_xpm);
-    list_pix = gdk_pixbuf_new_from_xpm_data ((const char**) playlist_xpm);
+    GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
+    folder_pix = gtk_icon_theme_load_icon (icon_theme,
+            "qgn_list_filesys_video_fldr", 26, (GtkIconLookupFlags)0, 0L);
+    auxiliary_pix = gtk_icon_theme_load_icon (icon_theme,
+            "qgn_list_filesys_common_fldr", 26, (GtkIconLookupFlags)0, 0L);
+    unknown_pix = gtk_icon_theme_load_icon (icon_theme,
+            "qgn_list_gene_unknown_file", 26, (GtkIconLookupFlags)0, 0L);
+    config_pix = gtk_icon_theme_load_icon (icon_theme,
+            "qgn_list_cp_soundset", 26, (GtkIconLookupFlags)0, 0L);
+    music_pix = gtk_icon_theme_load_icon (icon_theme,
+            "qgn_list_musicpl_file", 26, (GtkIconLookupFlags)0, 0L);
+    video_pix = gtk_icon_theme_load_icon (icon_theme,
+            "qgn_list_videopl_file", 26, (GtkIconLookupFlags)0, 0L);
+    video_link_pix = gtk_icon_theme_load_icon (icon_theme,
+            "qgn_list_musicpl_file_link", 26, (GtkIconLookupFlags)0, 0L);
+    list_pix = gtk_icon_theme_load_icon (icon_theme,
+            "qgn_list_medi", 26, (GtkIconLookupFlags)0, 0L);
 
     store = gtk_tree_store_new (3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_INT);
     view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
@@ -508,7 +581,7 @@ GtkWidget * createTreeView (Application * app) {
     //g_object_set_property (G_OBJECT (renderer), "editable", &prop_value);
     g_object_set (G_OBJECT (view), "headers-visible", FALSE, 0L);
     g_object_set (G_OBJECT (app->playlist_cell_renderer),
-            "font", "Sans 14", 0L);
+            "font", "Sans 15", 0L);
     g_signal_connect (G_OBJECT (app->playlist_cell_renderer),
             "edited", G_CALLBACK (cb_treecel_edited), app);
     gtk_tree_view_column_pack_start(col, app->playlist_cell_renderer, TRUE);
