@@ -23,6 +23,7 @@
 #include <signal.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stdarg.h>
 
 #define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus.h>
@@ -35,6 +36,28 @@
 
 using namespace KMPlayer;
 
+static void dbusSend (DBusConnection *conn,
+        const char *dest, const char *path, const char *iface,
+        const char *func, int first_arg_type, ...) {
+    va_list var_args;
+    DBusMessage *msg = dbus_message_new_method_call (
+            dest,
+            path,
+            iface,
+            func);
+    if (first_arg_type != DBUS_TYPE_INVALID) {
+        va_start (var_args, first_arg_type);
+        dbus_message_append_args_valist (msg, first_arg_type, var_args);
+        va_end (var_args);
+    }
+    dbus_message_set_no_reply (msg, TRUE);
+    dbus_connection_send (conn, msg, NULL);
+    dbus_connection_flush (conn);
+    dbus_message_unref (msg);
+}
+
+//#define dbusSend(conn, dest, path, iface, func, first_arg_type, rest...) \
+    osso_rpc_run((osso_context_t*)conn, dest, path, iface, func, NULL, first_arg_type, ## rest);
 
 Process::Process (ProcessNotify * ctrl, const char * n, bool xv)
  : m_notify (ctrl), m_viewer (0L), m_name (n),
@@ -149,7 +172,7 @@ void Process::scheduledStateChanged () {
 
 KMPLAYER_NO_EXPORT bool Process::play (NodePtr _mrl) {
     m_mrl = _mrl;
-    Mrl * m = _mrl ? _mrl->mrl () : 0L;
+    Mrl *m = mrl ();
     m_url = m ? m->absolutePath () : String ();
     no_video = m ? m->audio_only : false;
     m_position = m_length = 0;
@@ -207,7 +230,8 @@ bool MPlayer::play (NodePtr node) {
         return pause ();
     wait_for_pause = false;
     setState (Buffering);
-    Mrl *mrl = node ? node->mrl () : 0L;
+    Process::play (node);
+    Mrl *mrl = this->mrl ();
     if (!mrl) {
         errorLog() << "Not a Mrl " << (node ? node->nodeName() :"null") << endl;
         setState (Ready);
@@ -218,7 +242,6 @@ bool MPlayer::play (NodePtr node) {
         setState (Ready);
         return false;
     }
-    Process::play (node);
     request_seek = NO_SEEK;
     String wd ("/home/user");
     URL url (strurl);
@@ -245,8 +268,8 @@ bool MPlayer::play (NodePtr node) {
     cmdline += String (buf);
     snprintf (buf, sizeof(buf), " -wid %u ", m_viewer);
     cmdline += String (buf);
-    if (m_mrl->mrl ()->repeat > 0)
-        cmdline += String (" -loop ") + String::number (m_mrl->mrl()->repeat+1);
+    if (mrl->repeat > 0)
+        cmdline += String (" -loop ") + String::number (mrl->repeat+1);
     cmdline += Char (' ');
     char *quoted = g_shell_quote ((const char *) strurl);
     cmdline += String (quoted);
@@ -508,7 +531,7 @@ void MPlayer::readEvent (GIOChannel * src) {
                         } else {
                             char * v = strstr (tok, "Video: no video");
                             if (v) {
-                                no_video = true;
+                                setHasVideo (false);
                                 handled = true;
                             }
                         }
@@ -571,6 +594,10 @@ bool MPlayer::sendCommand (const String & cmd) {
 
 //-----------------------------------------------------------------------------
 
+static const char *media_server = "com.nokia.osso_media_server";
+static const char *media_server_path = "/com/nokia/osso_media_server";
+static const char *media_server_imusic = "com.nokia.osso_media_server.music";
+static const char *media_server_ivideo = "com.nokia.osso_media_server.video";
 static const char *media_server_filter = "type='signal',sender='com.nokia.osso_media_server'";
 
 gboolean cb_stoppedTimer (void * p) {
@@ -659,7 +686,8 @@ static DBusHandlerResult filter_func (DBusConnection * connection,
                     const char * info;
                     dbus_message_iter_get_basic (&iter, &info);
                     printf ("iradio_update %s\n", info);
-                    process->processNotify ()->infoMsg (info);
+                    process->processNotify ()->infoMsg (
+                            String (info) + process->stream_name);
                 } else
                     printf ("iradio_update %d\n", type);
             } else if (!strcmp (method, "details_received")) {
@@ -699,6 +727,15 @@ static DBusHandlerResult filter_func (DBusConnection * connection,
                                 } else if (key == "length") {
                                     int len = value.toInt () / 100;
                                     process->setLength (len);
+                                } else if (key == "has_video") {
+                                    int b = value.toInt ();
+                                    process->setHasVideo (!!b);
+                                } else if (key == "name") {
+                                    String s ("<hr><i>");
+                                    s += value;
+                                    s += "</i>";
+                                    process->stream_name = s;
+                                    process->processNotify ()->infoMsg (s);
                                 }
                             }
                         } else
@@ -834,10 +871,10 @@ static gboolean cb_getPosition (void * p) {
         DBusConnection * con = (DBusConnection *)osso_get_dbus_connection (
                 proc->ossoContext ());
         DBusMessage *msg = dbus_message_new_method_call (0L,
-                "/com/nokia/osso_media_server",
+                media_server_path,
                 proc->currentMediaServer (),
                 "get_position");
-        dbus_message_set_destination (msg, "com.nokia.osso_media_server");
+        dbus_message_set_destination (msg, media_server);
         if (dbus_connection_send_with_reply(con, msg, &proc->pos_pending, 5000))
             dbus_pending_call_set_notify (proc->pos_pending, cb_posMedia, p, 0);
         dbus_message_unref (msg);
@@ -852,6 +889,7 @@ OssoMediaServer::OssoMediaServer (ProcessNotify * ctrl, osso_context_t * ctx, bo
     : Process (ctrl, myname, xv),
       play_pending (0L), pos_pending (0L), seek_pending (0L),
       m_osso_context (ctx),
+      m_volume (0.3),
       current_service (""),
       dbus_filter (NULL),
       have_played (false),
@@ -894,7 +932,7 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::stop () {
     }
     if (!playing ()) return true;
     DBusConnection * connection = (DBusConnection *)osso_get_dbus_connection (m_osso_context);
-    DBusMessage *message = dbus_message_new_method_call ("com.nokia.osso_media_server", "/com/nokia/osso_media_server", current_service, "stop");
+    DBusMessage *message = dbus_message_new_method_call (media_server, media_server_path, current_service, "stop");
     DBusMessage *reply;
     DBusError error;
     dbus_error_init (&error);
@@ -929,11 +967,13 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::ready (unsigned int viewer) {
 }
 
 KMPLAYER_NO_EXPORT bool OssoMediaServer::retry () {
-    if (!current_service || !strcmp (current_service, "com.nokia.osso_media_server.music")) {
+    if (have_played_stream ||
+            !current_service ||
+            !strcmp (current_service, media_server_imusic)) {
         return false;
     } else if (retry_timer) {
         return true;
-    } else if (m_mrl && !m_mrl->mrl ()->audio_only && m_mrl->mrl ()->media_object) {
+    } else if (m_mrl && !mrl ()->audio_only && mrl ()->media_object) {
         try_audio_only = true;
         retry_timer = g_timeout_add (100, cb_retryAudioOnly, this);
         return true;
@@ -947,7 +987,7 @@ KMPLAYER_NO_EXPORT void OssoMediaServer::updateVideoSize () {
             (!has_xv && m_rect == m_cur_rect))
         return;
     DBusConnection * connection = (DBusConnection *)osso_get_dbus_connection (m_osso_context);
-    DBusMessage *message = dbus_message_new_method_call ("com.nokia.osso_media_server", "/com/nokia/osso_media_server", "com.nokia.osso_media_server.video", "set_video_window");
+    DBusMessage *message = dbus_message_new_method_call (media_server, media_server_path, media_server_ivideo, "set_video_window");
     dbus_message_set_auto_start (message, true);
     if (has_xv) {
         unsigned x = m_viewer;
@@ -973,15 +1013,17 @@ KMPLAYER_NO_EXPORT void OssoMediaServer::updateVideoSize () {
 }
 
 KMPLAYER_NO_EXPORT bool OssoMediaServer::play (NodePtr node) {
-    Mrl * mrl = node->mrl ();
     //debugLog() << "OssoMediaServer::play " << m_viewer << endl;
     if (retry_timer) // are we called by the retry timer
         retry_timer = 0;
     else
         try_audio_only = false;
+    Process::play (node);
+    Mrl *mrl = this->mrl ();
     if (!mrl)
         return false;
     String abspath = mrl->absolutePath ();
+    stream_name.clear ();
     if (abspath.isEmpty ()) {
         errorLog () << "empty path " << mrl->nodeName () << " " << mrl->src << endl;
         if (!mrl->src.isEmpty ()) // bug in absolutePath
@@ -989,7 +1031,7 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::play (NodePtr node) {
         else
             return false;
     }
-    Process::play (node);
+    have_played_stream = false;
     DBusConnection * connection = (DBusConnection *)osso_get_dbus_connection (m_osso_context);
     if (m_state < Buffering) {
         have_played = true;
@@ -1009,15 +1051,16 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::play (NodePtr node) {
             first_time = false;
         }
     }
+    volume (0, false);
     no_video = mrl->audio_only || try_audio_only;
     current_service = no_video
-        ? "com.nokia.osso_media_server.music"
-        : "com.nokia.osso_media_server.video";
-    DBusMessage *message = dbus_message_new_method_call (0L, "/com/nokia/osso_media_server", current_service, "play_media");
-    dbus_message_set_destination (message, "com.nokia.osso_media_server");
+        ? media_server_imusic
+        : media_server_ivideo;
+    DBusMessage *msg = dbus_message_new_method_call (0L, media_server_path, current_service, "play_media");
+    dbus_message_set_destination (msg, media_server);
     const char * path = (const char *) abspath;
     debugLog () << "OssoMediaServer::play " << current_service << " " << path << endl;
-    dbus_message_append_args(message, DBUS_TYPE_STRING,&path,DBUS_TYPE_INVALID);
+    dbus_message_append_args(msg, DBUS_TYPE_STRING,&path,DBUS_TYPE_INVALID);
     //dbus_connection_send (connection, message, NULL);
     //dbus_connection_flush (connection);
 
@@ -1026,11 +1069,11 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::play (NodePtr node) {
         setState (Buffering); // TODO: add timeout 
 
     if (dbus_connection_send_with_reply (connection,
-                message, &play_pending, 5000))
+                msg, &play_pending, 5000))
         dbus_pending_call_set_notify (play_pending,cb_playMedia,(void*)this,0L);
     else
         setState (Ready);
-    dbus_message_unref (message);
+    dbus_message_unref (msg);
     return m_state == Buffering;
 }
 
@@ -1039,12 +1082,9 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::pause () {
     if (m_state > Ready) {
         if (!has_xv)
             updateVideoSize ();
-        DBusConnection * connection = (DBusConnection *)osso_get_dbus_connection (m_osso_context);
-        DBusMessage *message = dbus_message_new_method_call (NULL, "/com/nokia/osso_media_server", "com.nokia.osso_media_server.video", "pause");
-        dbus_message_set_destination (message, "com.nokia.osso_media_server");
-        dbus_connection_send (connection, message, NULL);
-        dbus_connection_flush (connection);
-        dbus_message_unref (message);
+        dbusSend ((DBusConnection *)osso_get_dbus_connection (ossoContext ()),
+                media_server, media_server_path, media_server_ivideo,
+                "pause", DBUS_TYPE_INVALID);
     }
 }
 
@@ -1055,8 +1095,8 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::seek (int pos, bool absolute) {
     DBusConnection * con = (DBusConnection *)osso_get_dbus_connection (
             ossoContext ());
     DBusMessage *msg = dbus_message_new_method_call (0L,
-            "/com/nokia/osso_media_server", currentMediaServer (), "seek");
-    dbus_message_set_destination (msg, "com.nokia.osso_media_server");
+            media_server_path, currentMediaServer (), "seek");
+    dbus_message_set_destination (msg, media_server);
     int abs = absolute ? 1 : 0;
     dbus_message_append_args (msg, DBUS_TYPE_INT32, &abs,
             DBUS_TYPE_INT32, &pos, DBUS_TYPE_INVALID);
@@ -1067,6 +1107,20 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::seek (int pos, bool absolute) {
 }
 
 KMPLAYER_NO_EXPORT bool OssoMediaServer::volume (int incdec, bool absolute) {
+    m_volume += 1.0 * incdec / 40;
+    if (m_volume < 0.0)
+        m_volume = 0.0;
+    else if (m_volume > 1.0)
+        m_volume = 1.0;
+
+    DBusConnection *conn = (DBusConnection *)osso_get_dbus_connection (
+                            ossoContext ());
+    dbusSend (conn, media_server, media_server_path, media_server_imusic,
+            "set_volume",
+            DBUS_TYPE_DOUBLE, &m_volume, DBUS_TYPE_INVALID);
+    dbusSend (conn, media_server, media_server_path, media_server_ivideo,
+            "set_volume",
+            DBUS_TYPE_DOUBLE, &m_volume, DBUS_TYPE_INVALID);
 }
 
 KMPLAYER_NO_EXPORT bool OssoMediaServer::saturation (int val, bool absolute) {
@@ -1113,13 +1167,12 @@ KMPLAYER_NO_EXPORT void OssoMediaServer::processStopped () {
 KMPLAYER_NO_EXPORT void OssoMediaServer::processBuffering (int perc) {
     if (perc < 100)
         setState (Buffering);
-    else
-        setState (Playing);
     m_notify->setLoading (perc);
 }
 
 KMPLAYER_NO_EXPORT void OssoMediaServer::processPlaying () {
     setState (Playing);
+    have_played_stream = true;
     if (!pos_timer)
         pos_timer = g_timeout_add (5000, cb_getPosition, this);
 }
@@ -1285,7 +1338,8 @@ KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
     if (playing ())
         return true;
     setState (Buffering);
-    Mrl *mrl = node ? node->mrl () : 0L;
+    Process::play (node);
+    Mrl *mrl = this->mrl ();
     if (!mrl) {
         errorLog() << "Not a Mrl " << (node ? node->nodeName() :"null") << endl;
         setState (Ready);
@@ -1297,7 +1351,6 @@ KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
         return false;
     }
     streams = g_tree_new (streamCompare);
-    Process::play (node);
 
     GTree *params = g_tree_new ((GCompareFunc)::strcmp);
     if (mrl->id == id_node_html_object) {
