@@ -52,7 +52,7 @@ public:
     //virtual bool grabPicture (const KURL & url, int pos);
     //bool supports (const char * source) const;
     State state () const { return m_state; }
-    NodePtr mrl () const { return m_mrl; }
+    Mrl *mrl () { return m_mrl ? m_mrl->mrl () : NULL; }
 //signals:
     //void grabReady (const QString & path);
 public:// slots:
@@ -79,6 +79,7 @@ public:// slots:
     void setHeight (int h) { m_height = h; }
     void updateAspects ();
     bool hasVideo () const { return playing () && !no_video; }
+    void setHasVideo (bool b) { no_video = !b; }
     bool hasXv () const { return has_xv; }
 protected:
     //Source * m_source;
@@ -188,15 +189,18 @@ public: //slots:
     DBusPendingCall * play_pending;
     DBusPendingCall * pos_pending;
     DBusPendingCall * seek_pending;
+    String stream_name;
     static const char *myname;
     void updateVideoSize ();
 private:
     osso_context_t * m_osso_context;
     Rect m_rect;  // see setVideoWindow (..)
     Rect m_cur_rect; // last real setting
+    double m_volume;
     const char *current_service;
     const char *dbus_filter;
     bool have_played;
+    bool have_played_stream;
     bool try_audio_only;
     bool first_time;
     int retry_timer;
