@@ -330,6 +330,7 @@ GnomeVFSFile::GnomeVFSFile (const String & p) : d (new GnomeVFSFilePrivate) {
 
 KMPLAYER_NO_CDTOR_EXPORT GnomeVFSFile::~GnomeVFSFile () {
     close ();
+    delete d;
 }
 
 KMPLAYER_NO_EXPORT bool GnomeVFSFile::open (FileMode mode) {
@@ -468,7 +469,10 @@ KMPLAYER_NO_EXPORT String GnomeVFSUrl::protocol () const {
 KMPLAYER_NO_EXPORT String GnomeVFSUrl::decode_string (const String & s) {
     if (s.isEmpty ())
         return String ();
-    return String (gnome_vfs_unescape_string ((const char *) s, ""));
+    String str;
+    GLibString &gs = str;
+    gs.take(gnome_vfs_unescape_string ((const char *) s, ""));
+    return str;
 }
 
 KMPLAYER_NO_EXPORT bool GnomeVFSUrl::isLocalFile () const {
