@@ -119,26 +119,6 @@ public:
     operator double () const { return 1.0 * value / 256; }
 };
 
-/**                                   a  b  0
- * Matrix for coordinate transforms   c  d  0
- *                                    tx ty 1     */
-class KMPLAYER_NO_EXPORT Matrix {
-    friend class SizeEvent;
-    float a, b, c, d;
-    Single tx, ty; 
-public:
-    Matrix ();
-    Matrix (const Matrix & matrix);
-    Matrix (Single xoff, Single yoff, float xscale, float yscale);
-    void getXY (Single & x, Single & y) const;
-    void getXYWH (Single & x, Single & y, Single & w, Single & h) const;
-    void invXYWH (Single & x, Single & y, Single & w, Single & h) const;
-    void transform (const Matrix & matrix);
-    void scale (float sx, float sy);
-    void translate (Single x, Single y);
-    // void rotate (float phi); // add this when needed
-};
-
 class KMPLAYER_NO_EXPORT SRect {
     Single _x, _y, _w, _h;
 public:
@@ -168,7 +148,7 @@ public:
     IRect unite (const IRect & r) const;
     IRect intersect (const IRect & r) const;
     bool isValid () const { return w >= 0 && h >= 0; }
-    bool isEmpty () const { return w < 0 || h < 0; }
+    bool isEmpty () const { return w <= 0 || h <= 0; }
 };
 
 template <class T>
