@@ -511,7 +511,7 @@ static bool nsInvokeDefault (NPP instance, NPObject * npobj,
     return npobj->_class->invokeDefault (npobj,args, arg_count, result);
 }
 
-static bool nsEvaluate (NPP instance, NPObject * npobj, NPString * script,
+static bool doEvaluate (NPP instance, NPObject * npobj, NPString * script,
         NPVariant * result) {
     char * this_var;
     char * this_var_type;
@@ -563,6 +563,27 @@ static bool nsEvaluate (NPP instance, NPObject * npobj, NPString * script,
     return true;
 }
 
+static bool nsEvaluate (NPP instance, NPObject * npobj, NPString * script,
+        NPVariant * result) {
+    NPString str;
+    char *jsscript;
+    char *escaped;
+    bool res;
+
+    escaped = g_strescape (script->utf8characters, "");
+    str.utf8length = strlen (escaped) + 9;
+    jsscript = (char *) malloc (str.utf8length);
+    sprintf (jsscript, "eval(\"%s\")", escaped);
+    str.utf8characters = jsscript;
+
+    res = doEvaluate (instance, npobj, &str, result);
+
+    free (jsscript);
+    g_free (escaped);
+
+    return res;
+}
+
 static bool nsGetProperty (NPP instance, NPObject * npobj,
         NPIdentifier property, NPVariant * result) {
     (void)instance;
@@ -704,7 +725,7 @@ static bool windowClassInvoke (NPObject *npobj, NPIdentifier method,
 
     str.utf8characters = buf;
     str.utf8length = pos;
-    res = nsEvaluate (npp, npobj, &str, result);
+    res = doEvaluate (npp, npobj, &str, result);
 
     return true;
 }
@@ -747,7 +768,7 @@ static bool windowClassGetProperty (NPObject *npobj, NPIdentifier property,
     jo.parent = (JsObject *) npobj;
     createJsName (&jo, (char **)&fullname.utf8characters, &fullname.utf8length);
 
-    res = nsEvaluate (npp, npobj, &fullname, result);
+    res = doEvaluate (npp, npobj, &fullname, result);
 
     free ((char *) fullname.utf8characters);
 
