@@ -24,6 +24,7 @@
 #include <cairo-xlib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
+//#include <X11/extensions/Xdbe.h>
 
 #include "kmplayertypes.h"
 #include "kmplayer_smil.h"
@@ -38,35 +39,136 @@ using namespace KMPlayer;
 
 //-------------------------------------------------------------------------
 
-static void copyImage (Surface *s, int w, int h, Image *img, cairo_surface_t * similar) {
-    int iw = img->width ();
-    int ih = img->height ();
+namespace {
+/**                                   a  b  0
+ * Matrix for coordinate transforms   c  d  0
+ *                                    tx ty 1     */
+class KMPLAYER_NO_EXPORT Matrix {
+    float a, b, c, d;
+    Single tx, ty;
+public:
+    Matrix ();
+    Matrix (const Matrix & matrix);
+    Matrix (Single xoff, Single yoff, float xscale, float yscale);
+    void getXY (Single & x, Single & y) const;
+    void getXYWH (Single & x, Single & y, Single & w, Single & h) const;
+    void invXYWH (Single & x, Single & y, Single & w, Single & h) const;
+    void transform (const Matrix & matrix);
+    void scale (float sx, float sy);
+    void translate (Single x, Single y);
+    // void rotate (float phi); // add this when needed
+};
 
-    const GPixbuf &gpb = (const GPixbuf) *img;
-    int channels;
-    unsigned char *bits = pixbufBits (gpb.pixbuf (), channels);
-    cairo_surface_t *sf = cairo_image_surface_create_for_data (
-            bits, channels == 4 ? CAIRO_FORMAT_ARGB32 : CAIRO_FORMAT_RGB24,
-            iw, ih, iw*4);
-    cairo_pattern_t *img_pat = cairo_pattern_create_for_surface (sf);
+}
+
+Matrix::Matrix () : a (1.0), b (0.0), c (0.0), d (1.0), tx (0), ty (0) {}
+
+Matrix::Matrix (const Matrix & m)
+ : a (m.a), b (m.b), c (m.c), d (m.d), tx (m.tx), ty (m.ty) {}
+
+Matrix::Matrix (Single xoff, Single yoff, float xscale, float yscale)
+ : a (xscale), b (0.0), c (0.0), d (yscale), tx (xoff), ty (yoff) {}
+
+void Matrix::getXY (Single & x, Single & y) const {
+    x = Single (x * a) + tx;
+    y = Single (y * d) + ty;
+}
+
+void Matrix::getXYWH (Single & x, Single & y, Single & w, Single & h) const {
+    getXY (x, y);
+    w *= a;
+    h *= d;
+}
+
+void Matrix::invXYWH (Single & x, Single & y, Single & w, Single & h) const {
+    if (a > 0.00001 && d > 0.00001) {
+        w /= a;
+        h /= d;
+        x = Single ((x - tx) / a);
+        y = Single ((y - ty) / d);
+    } else {
+        warningLog () << "Not invering " << a << ", " << d << " scale" << endl;
+    }
+}
+
+void Matrix::transform (const Matrix & matrix) {
+    // TODO: rotate
+    a *= matrix.a;
+    d *= matrix.d;
+    tx = Single (tx * matrix.a) + matrix.tx;
+    ty = Single (ty * matrix.d) + matrix.ty;
+}
+
+void Matrix::scale (float sx, float sy) {
+    a *= sx;
+    d *= sy;
+    tx *= sx;
+    ty *= sy;
+}
+
+void Matrix::translate (Single x, Single y) {
+    tx += x;
+    ty += y;
+}
+
+//-----------------------------------------------------------------------------
+
+void ImageData::copyImage (Surface *s, int w, int h, cairo_surface_t *similar) {
+    cairo_surface_t *src_sf;
+
+    unsigned char *bits = NULL;
+    if (surface) {
+        src_sf = surface;
+    } else {
+        const GPixbuf &gpb = (const GPixbuf) *image;
+        int channels;
+        unsigned char *bits = pixbufBits (gpb.pixbuf (), channels);
+        has_alpha = 4 == channels;
+        src_sf = cairo_image_surface_create_for_data (
+                bits, has_alpha ? CAIRO_FORMAT_ARGB32 : CAIRO_FORMAT_RGB24,
+                width, height, width*4);
+        if (flags & ImagePixmap && !(flags & ImageAnimated)) {
+            surface = cairo_surface_create_similar (similar,
+                    has_alpha ? CAIRO_CONTENT_COLOR_ALPHA : CAIRO_CONTENT_COLOR,
+                    width, height);
+            cairo_pattern_t *pat = cairo_pattern_create_for_surface (src_sf);
+            cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
+            cairo_t *cr = cairo_create (surface);
+            cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+            cairo_set_source (cr, pat);
+            cairo_paint (cr);
+            cairo_destroy (cr);
+            cairo_pattern_destroy (pat);
+            cairo_surface_destroy (src_sf);
+            g_free (bits);
+            src_sf = surface;
+            delete image;
+            image = NULL;
+        }
+    }
+
+    cairo_pattern_t *img_pat = cairo_pattern_create_for_surface (src_sf);
     cairo_pattern_set_extend (img_pat, CAIRO_EXTEND_NONE);
-    if (w != iw && h != ih) {
+    if (w != width && h != height) {
         cairo_matrix_t mat;
-        cairo_matrix_init_scale (&mat, 1.0 * iw/w, 1.0 * ih/h);
+        cairo_matrix_init_scale (&mat, 1.0 * width/w, 1.0 * height/h);
         cairo_pattern_set_matrix (img_pat, &mat);
     }
     if (!s->surface)
         s->surface = cairo_surface_create_similar (similar,
-                channels == 4 ?
-                CAIRO_CONTENT_COLOR_ALPHA : CAIRO_CONTENT_COLOR, w, h);
+                has_alpha ?  CAIRO_CONTENT_COLOR_ALPHA : CAIRO_CONTENT_COLOR, w, h);
     cairo_t *cr = cairo_create (s->surface);
     cairo_set_source (cr, img_pat);
+    if (!has_alpha)
+        cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
     cairo_paint (cr);
     cairo_destroy (cr);
 
     cairo_pattern_destroy (img_pat);
-    cairo_surface_destroy (sf);
-    g_free (bits);
+    if (!surface) {
+        cairo_surface_destroy (src_sf);
+        g_free (bits);
+    }
 }
 
 //-------------------------------------------------------------------------
@@ -83,13 +185,14 @@ public:
 
     SurfacePtr createSurface (NodePtr owner, const SRect & rect);
     IRect toScreen (Single x, Single y, Single w, Single h);
+    IRect clipToScreen (Single x, Single y, Single w, Single h);
     void resize (const SRect & rect);
     void repaint ();
     void repaint (const SRect &rect);
     void video (Mrl *mt);
 
     NodePtrW current_video;
-    ViewArea * view_widget;
+    ViewArea *view_widget;
 };
 
 } // namespace
@@ -116,8 +219,6 @@ SurfacePtr ViewSurface::createSurface (NodePtr owner, const SRect & rect) {
 
 KDE_NO_EXPORT void ViewSurface::resize (const SRect &r) {
     bounds = r;
-    if (surface)
-        cairo_xlib_surface_set_size (surface, (int)r.width(), (int)r.height ());
     /*if (rect == nrect)
         ;//return;
     SRect pr = rect.unite (nrect); // for repaint
@@ -135,15 +236,31 @@ KDE_NO_EXPORT IRect ViewSurface::toScreen (Single x, Single y, Single w, Single
     return IRect (x, y, w, h);
 }
 
+KDE_NO_EXPORT IRect ViewSurface::clipToScreen (Single x, Single y, Single w, Single h) {
+    Matrix m (0, 0, xscale, yscale);
+    m.translate (bounds.x (), bounds.y ());
+    m.getXYWH (x, y, w, h);
+    SRect r = bounds.intersect (SRect (x, y, w, h));
+    x= r.x (), y = r.y (), w = r.width (), h = r.height ();
+    for (SurfacePtr s = parentNode (); s; s = s->parentNode ()) {
+        m = Matrix (0, 0, s->xscale, s->yscale);
+        m.translate (s->bounds.x (), s->bounds.y ());
+        m.getXYWH (x, y, w, h);
+        r = SRect (s->bounds).intersect (SRect (x, y, w, h));
+        x= r.x (); y = r.y (); w = r.width (); h = r.height ();
+    }
+    return IRect (x, y, w, h);
+}
+
 KDE_NO_EXPORT void ViewSurface::repaint (const SRect &r) {
     markDirty ();
-    view_widget->scheduleRepaint (toScreen (r.x (), r.y (), r.width (), r.height ()));
+    view_widget->scheduleRepaint (clipToScreen (r.x (), r.y (), r.width (), r.height ()));
     //kdDebug() << "Surface::repaint x:" << (int)x << " y:" << (int)y << " w:" << (int)w << " h:" << (int)h << endl;
 }
 
 KDE_NO_EXPORT void ViewSurface::repaint () {
     markDirty ();
-    view_widget->scheduleRepaint (toScreen (0, 0, bounds.width (), bounds.height ()));
+    view_widget->scheduleRepaint (clipToScreen (0, 0, bounds.width (), bounds.height ()));
 }
 
 KDE_NO_EXPORT void ViewSurface::video (Mrl *m) {
@@ -159,22 +276,6 @@ KDE_NO_EXPORT void ViewSurface::video (Mrl *m) {
 
 //-------------------------------------------------------------------------
 
-static cairo_surface_t * cairoCreateSurface (GtkWidget * wid, int w, int h) {
-    GdkDisplay * gdisplay = gtk_widget_get_display (wid);
-    Display * display = gdk_x11_display_get_xdisplay (gdisplay);
-    Window id = gdk_x11_drawable_get_xid (GDK_DRAWABLE (wid->window));
-    return cairo_xlib_surface_create (display, id,
-            DefaultVisual (display, DefaultScreen (display)), w, h);
-    /*return cairo_xlib_surface_create_with_xrender_format (
-            qt_xdisplay (),
-            id,
-            DefaultScreenOfDisplay (qt_xdisplay ()),
-            XRenderFindVisualFormat (qt_xdisplay (),
-                DefaultVisual (qt_xdisplay (),
-                    DefaultScreen (qt_xdisplay ()))),
-            w, h);*/
-}
-
 #define CAIRO_SET_SOURCE_RGB(cr,c)           \
     cairo_set_source_rgb ((cr),               \
             1.0 * (((c) >> 16) & 0xff) / 255, \
@@ -205,6 +306,7 @@ public:
     ~CairoPaintVisitor ();
     using Visitor::visit;
     void visit (Node * n);
+    void visit (SMIL::Smil *);
     void visit (SMIL::Layout *);
     void visit (SMIL::Region *);
     void visit (SMIL::Transition *);
@@ -235,7 +337,7 @@ CairoPaintVisitor::CairoPaintVisitor (cairo_surface_t * cs, Matrix m,
         cairo_surface = cs;
     cr = cairo_create (cairo_surface);
     if (toplevel) {
-        //cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+        cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
         cairo_set_tolerance (cr, 0.5 );
         cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
         cairo_rectangle (cr, 0, 0, rect.w, rect.h);
@@ -247,11 +349,10 @@ CairoPaintVisitor::CairoPaintVisitor (cairo_surface_t * cs, Matrix m,
     cr = cairo_create (cs);
     if (toplevel) {
         cairo_rectangle (cr, rect.x, rect.y, rect.w, rect.h);
-        cairo_clip (cr);
-        //cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+        cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
         cairo_set_tolerance (cr, 0.5 );
         cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
-        cairo_push_group (cr);
+        //cairo_push_group (cr);
         cairo_set_source_rgb (cr, 0, 0, 0);
         cairo_rectangle (cr, rect.x, rect.y, rect.w, rect.h);
         cairo_fill (cr);
@@ -266,8 +367,8 @@ CairoPaintVisitor::CairoPaintVisitor (cairo_surface_t * cs, Matrix m,
 }
 
 KDE_NO_CDTOR_EXPORT CairoPaintVisitor::~CairoPaintVisitor () {
-    if (toplevel) {
 #if USE_IMG_SURFACE
+    if (toplevel) {
         cairo_pattern_t *pat = cairo_pattern_create_for_surface (cairo_surface);
         cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
         cairo_matrix_t mat;
@@ -280,14 +381,8 @@ KDE_NO_CDTOR_EXPORT CairoPaintVisitor::~CairoPaintVisitor () {
         cairo_destroy (wcr);
         cairo_pattern_destroy (pat);
         cairo_surface_destroy (cairo_surface);
-#else
-        cairo_pattern_t * pat = cairo_pop_group (cr);
-        cairo_set_source (cr, pat);
-        cairo_rectangle (cr, clip.x, clip.y, clip.w, clip.h);
-        cairo_fill (cr);
-        cairo_pattern_destroy (pat);
-#endif
     }
+#endif
     cairo_destroy (cr);
 }
 
@@ -295,6 +390,11 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (Node * n) {
     warningLog() << "Paint called on " << n->nodeName() << endl;
 }
 
+KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Smil *s) {
+    if (s->active () && s->layout_node)
+        s->layout_node->accept (this);
+}
+
 KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (SMIL::RegionBase * reg) {
     // next visit listeners
     NodeRefListPtr nl = reg->listeners (mediatype_attached);
@@ -325,36 +425,31 @@ KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (SMIL::RegionBase * reg) {
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Layout * reg) {
     //debugLog() << "Visit " << reg->nodeName() << endl;
     SMIL::RegionBase *rb = convertNode <SMIL::RegionBase> (reg->root_layout);
-    if (reg->surface () && rb) {
+    Surface *s = reg->surface ();
+    if (s && rb) {
         //cairo_save (cr);
         Matrix m = matrix;
 
-        SRect rect = reg->region_surface->bounds;
+        SRect rect = s->bounds;
         Single x, y, w = rect.width(), h = rect.height();
         matrix.getXYWH (x, y, w, h);
 
         IRect clip_save = clip;
         clip = clip.intersect (IRect (x, y, w, h));
 
-        rb->region_surface = reg->region_surface;
-        rb->region_surface->background_color = rb->background_color;
-
-        if (reg->region_surface->background_color & 0xff000000) {
-            CAIRO_SET_SOURCE_RGB (cr, reg->region_surface->background_color);
+        s->background_color = rb->background_color;
+        if (s->background_color & 0xff000000) {
+            CAIRO_SET_SOURCE_RGB (cr, s->background_color);
             cairo_rectangle (cr, clip.x, clip.y, clip.w, clip.h);
             cairo_fill (cr);
         }
-        //cairo_rectangle (cr, xoff, yoff, w, h);
-        //cairo_clip (cr);
 
-        matrix = Matrix (0, 0, reg->region_surface->xscale, reg->region_surface->yscale);
+        matrix = Matrix (0, 0, s->xscale, s->yscale);
         matrix.transform (m);
         traverseRegion (reg);
         //cairo_restore (cr);
         matrix = m;
         clip = clip_save;
-
-        rb->region_surface = 0L;
     }
 }
 
@@ -373,10 +468,9 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Region * reg) {
         IRect clip_save = clip;
         clip = clip.intersect (IRect (x, y, w, h));
         cairo_save (cr);
-        Image *bg_img = reg->bg_image &&
-            !reg->bg_image->isEmpty()
-                ? reg->bg_image->cached_img->image
-                : NULL;
+        ImageData *bg_img = reg->bg_image && !reg->bg_image->isEmpty()
+            ? reg->bg_image->cached_img.ptr ()
+            : NULL;
         if ((SMIL::RegionBase::ShowAlways == reg->show_background ||
                     reg->m_AttachedMediaTypes->first ()) &&
                 (s->background_color & 0xff000000 || bg_img)) {
@@ -388,11 +482,13 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Region * reg) {
             }
             if (bg_img) {
                 Single x1, y1;
-                Single w = bg_img->width ();
-                Single h = bg_img->height();
+                Single w = bg_img->width;
+                Single h = bg_img->height;
                 matrix.getXYWH (x1, y1, w, h);
                 if (!s->surface)
-                    copyImage (s, w, h, bg_img, cairo_surface);
+                    bg_img->copyImage (s, w, h, cairo_surface);
+                if (bg_img->has_alpha)
+                    cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
                 cairo_pattern_t *pat = cairo_pattern_create_for_surface (s->surface);
                 cairo_pattern_set_extend (pat, CAIRO_EXTEND_REPEAT);
                 cairo_matrix_t mat;
@@ -402,6 +498,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Region * reg) {
                 cairo_rectangle (cr, clip.x, clip.y, clip.w, clip.h);
                 cairo_fill (cr);
                 cairo_pattern_destroy (pat);
+                if (bg_img->has_alpha)
+                    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
             }
             cairo_restore (cr);
         }
@@ -458,8 +556,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Transition *trans) {
         else //if (SMIL::Transition::SubFromLeft == trans->sub_type)
             dx = -(int) ((1.0 - perc) * clip.w);
         cairo_matrix_translate (&cur_mat, -dx, -dy);
-        IRect rect = clip.intersect (IRect (clip.x + dx, clip.y + dy,
-                    clip.w - dx, clip.h - dy));
+        IRect rect = clip.intersect (IRect (clip.x + dx, clip.y + dy, clip.w, clip.h));
         cairo_rectangle (cr, rect.x, rect.y, rect.w, rect.h);
         CAIRO_SET_PATTERN_COND(cr, cur_pat, cur_mat)
     } else if (SMIL::Transition::IrisWipe == trans->type) {
@@ -598,8 +695,11 @@ KDE_NO_EXPORT void CairoPaintVisitor::paint (SMIL::MediaType *mt, Surface *s,
     }
     opacity *= mt->opacity / 100.0;
     if (opacity < 0.99) {
+        cairo_operator_t op = cairo_get_operator (cr);
+        cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
         cairo_clip (cr);
         cairo_paint_with_alpha (cr, opacity);
+        cairo_set_operator (cr, op);
     } else {
         cairo_fill (cr);
     }
@@ -625,6 +725,7 @@ static Mrl *findActiveMrl (Node *n, bool *rp_or_smil) {
             if (m)
                 return m;
         }
+    return NULL;
 }
 
 KDE_NO_EXPORT
@@ -684,7 +785,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
     }
     ImageMedia *im = static_cast <ImageMedia *> (img->media_object);
     ImageData *id = im ? im->cached_img.ptr () : NULL;
-    if (!id || !id->image || img->width <= 0 || img->height <= 0) {
+    if (!id || im->isEmpty () || img->width <= 0 || img->height <= 0) {
         s->remove();
         return;
     }
@@ -698,8 +799,12 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
     if (clip_rect.isEmpty ())
         return;
     if (!s->surface || s->dirty)
-        copyImage (s, w, h, id->image, cairo_surface);
+        id->copyImage (s, w, h, cairo_surface);
+    if (id->has_alpha)
+        cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
     paint (img, s, x, y, clip_rect);
+    if (id->has_alpha)
+        cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
     s->dirty = false;
 }
 
@@ -813,6 +918,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
         cairo_t * cr_txt = cairo_create (s->surface);
         cairo_set_font_size (cr_txt, scale * txt->font_size);
         if (txt->bg_opacity) { // TODO real alpha
+            cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
             CAIRO_SET_SOURCE_RGB (cr_txt, txt->background_color);
             cairo_paint (cr_txt);
         }
@@ -868,15 +974,19 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Brush * brush) {
             cairo_rectangle (cr, (int) x, (int) y, (int) w, (int) h);
         }
         opacity *= brush->opacity / 100.0;
-        if (opacity < 0.99)
+        if (opacity < 0.99) {
+            cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
             cairo_set_source_rgba (cr,
                     1.0 * ((color >> 16) & 0xff) / 255,
                     1.0 * ((color >> 8) & 0xff) / 255,
                     1.0 * (color & 0xff) / 255,
                     opacity);
-        else
+        } else {
             CAIRO_SET_SOURCE_RGB (cr, color);
+        }
         cairo_fill (cr);
+        if (opacity < 0.99)
+            cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
         s->dirty = false;
         cairo_restore (cr);
     }
@@ -954,8 +1064,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Fadein * fi) {
                 sh = img->height;
             if ((int)fi->w && (int)fi->h && (int)sw && (int)sh) {
                 if (!img->img_surface->surface)
-                    copyImage (img->img_surface, img->width, img->height,
-                            im->cached_img->image, cairo_surface);
+                    im->cached_img->copyImage (img->img_surface,
+                            img->width, img->height, cairo_surface);
                 cairo_matrix_t matrix;
                 cairo_matrix_init_identity (&matrix);
                 float scalex = 1.0 * sw / fi->w;
@@ -1006,8 +1116,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Crossfade * cf) {
                 sh = img->height;
             if ((int)cf->w && (int)cf->h && (int)sw && (int)sh) {
                 if (!img->img_surface->surface)
-                    copyImage (img->img_surface, img->width, img->height,
-                            im->cached_img->image, cairo_surface);
+                    im->cached_img->copyImage (img->img_surface,
+                            img->width, img->height, cairo_surface);
                 cairo_save (cr);
                 cairo_matrix_t matrix;
                 cairo_matrix_init_identity (&matrix);
@@ -1067,8 +1177,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Wipe * wipe) {
 
             if ((int)w && (int)h) {
                 if (!img->img_surface->surface)
-                    copyImage (img->img_surface, img->width, img->height,
-                            im->cached_img->image, cairo_surface);
+                    im->cached_img->copyImage (img->img_surface,
+                            img->width, img->height, cairo_surface);
                 cairo_matrix_t matrix;
                 cairo_matrix_init_identity (&matrix);
                 float scalex = 1.0 * sw / wipe->w;
@@ -1133,6 +1243,7 @@ public:
     KDE_NO_CDTOR_EXPORT ~MouseVisitor () {}
     using Visitor::visit;
     void visit (Node * n);
+    void visit (SMIL::Smil *);
     void visit (SMIL::Layout *);
     void visit (SMIL::Region *);
     void visit (SMIL::TimedMrl * n);
@@ -1153,6 +1264,11 @@ KDE_NO_EXPORT void MouseVisitor::visit (Node * n) {
     debugLog () << "Mouse event ignored for " << n->nodeName () << endl;
 }
 
+KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Smil *s) {
+    if (s->active () && s->layout_node)
+        s->layout_node->accept (this);
+}
+
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Layout * layout) {
     if (layout->surface ()) {
         Matrix m = matrix;
@@ -1203,19 +1319,23 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Region * region) {
         int saved_event = event;
         if (node->active ()) {
             bool propagate_listeners = !child_handled;
+            bool pass_event = !child_handled;
             if (event == event_pointer_moved) {
-                propagate_listeners = true; // always pass move events
+                pass_event = true; // always pass move events
                 if (region->has_mouse && (!inside || child_handled)) {
+                    propagate_listeners = true;
                     region->has_mouse = false;
                     event = event_outbounds;
                 } else if (inside && !child_handled && !region->has_mouse) {
+                    propagate_listeners = true;
                     region->has_mouse = true;
                     event = event_inbounds;
                 }
             }// else // event_pointer_clicked
-            if (propagate_listeners) {
-                NodeRefListPtr nl = region->listeners (
-                        event == event_pointer_moved ? mediatype_attached : event);
+            if (propagate_listeners)
+                region->propagateEvent (new Event (region, event));
+            if (pass_event) {
+                NodeRefListPtr nl = region->listeners (mediatype_attached);
                 if (nl) {
                     for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ()) {
                         if (c->data)
@@ -1312,38 +1432,39 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::TimedMrl * timedmrl) {
     timedmrl->runtime ()->processEvent (event);
 }
 
-KDE_NO_EXPORT void MouseVisitor::visit (SMIL::MediaType * mediatype) {
-    if (mediatype->sensitivity == SMIL::MediaType::sens_transparent) {
+KDE_NO_EXPORT void MouseVisitor::visit (SMIL::MediaType *mt) {
+    if (mt->sensitivity == SMIL::MediaType::sens_transparent) {
         bubble_up = true;
         return;
     }
-    Surface *s = mediatype->surface ();
+    Surface *s = mt->surface ();
     if (!s)
         return;
-    if (s->node && s->node.ptr () != mediatype) {
+    if (s->node && s->node.ptr () != mt) {
         s->node->accept (this);
         return;
     }
     SRect rect = s->bounds;
     Single rx = rect.x(), ry = rect.y(), rw = rect.width(), rh = rect.height();
     matrix.getXYWH (rx, ry, rw, rh);
-    bool inside = x > rx && x < rx+rw && y > ry && y< ry+rh;
-    if (!inside && event == event_pointer_clicked)
-        return; // FIXME, also in/outbounds are bounds related
+    const bool inside = x > rx && x < rx+rw && y > ry && y< ry+rh;
+    if (!inside && (event == event_pointer_clicked || inside == mt->has_mouse))
+        return;
+    mt->has_mouse = inside;
 
-    NodeRefListPtr nl = mediatype->listeners (
+    NodeRefListPtr nl = mt->listeners (
             event == event_pointer_moved ? mediatype_attached : event);
     if (nl)
         for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ()) {
-            if (c->data && c->data.ptr () != mediatype)
+            if (c->data && c->data.ptr () != mt)
                 c->data->accept (this);
             if (!node->active ())
                 return;
         }
     if (event != event_pointer_moved)
-        visit (static_cast <SMIL::TimedMrl *> (mediatype));
+        visit (static_cast <SMIL::TimedMrl *> (mt));
     if (event != event_inbounds && event != event_outbounds) {
-      SMIL::RegionBase *r=convertNode<SMIL::RegionBase>(mediatype->region_node);
+      SMIL::RegionBase *r=convertNode<SMIL::RegionBase>(mt->region_node);
       if (r && r->surface () &&
               r->id != SMIL::id_node_smil &&
               r->region_surface->node && r != r->region_surface->node.ptr ())
@@ -1353,18 +1474,86 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::MediaType * mediatype) {
 
 //-----------------------------------------------------------------------------
 
+namespace KMPlayer {
+class KMPLAYER_NO_EXPORT ViewAreaPrivate {
+public:
+    ViewAreaPrivate (ViewArea *v)
+        : m_view_area (v), dpy (NULL), backing_store (0) {}
+
+    ~ViewAreaPrivate() {
+        destroyBackingStore ();
+    }
+
+    Display *display () {
+        if (!dpy)
+            dpy = gdk_x11_display_get_xdisplay (gtk_widget_get_display (
+                        m_view_area->viewWidget ()));
+        return dpy;
+    }
+    void resizeSurface (Surface *s) {
+        int w = (int) s->bounds.width ();
+        int h = (int) s->bounds.height ();
+        if (s->surface) {
+                //cairo_xlib_surface_set_size (s->surface, w, h);
+            Window wid = gdk_x11_drawable_get_xid (
+                    GDK_DRAWABLE (m_view_area->viewWidget ()->window));
+            XFreePixmap (display (), backing_store);
+            backing_store = XCreatePixmap (dpy, wid, w, h,
+                    gdk_drawable_get_depth (
+                        GDK_DRAWABLE (m_view_area->viewWidget ()->window)));
+            cairo_xlib_surface_set_drawable(s->surface, backing_store, w,h);
+        }
+    }
+    cairo_surface_t *createSurface (int w, int h) {
+        Window wid = gdk_x11_drawable_get_xid (
+                GDK_DRAWABLE (m_view_area->viewWidget ()->window));
+        Display *d = display ();
+        backing_store = XCreatePixmap (d, wid, w, h, gdk_drawable_get_depth (
+                    GDK_DRAWABLE (m_view_area->viewWidget ()->window)));
+        return cairo_xlib_surface_create (d, backing_store,
+                DefaultVisual (d, DefaultScreen (d)), w, h);
+        /*return cairo_xlib_surface_create_with_xrender_format (
+            QX11Info::display (),
+            id,
+            DefaultScreenOfDisplay (QX11Info::display ()),
+            XRenderFindVisualFormat (QX11Info::display (),
+                DefaultVisual (QX11Info::display (),
+                    DefaultScreen (QX11Info::display ()))),
+            w, h);*/
+    }
+    void swapBuffer (int sx, int sy, int sw, int sh, int dx, int dy) {
+        Window wid = gdk_x11_drawable_get_xid (
+                GDK_DRAWABLE (m_view_area->viewWidget ()->window));
+        GC gc = XCreateGC (display (), backing_store, 0, NULL);
+        XCopyArea (dpy, backing_store, wid, gc, sx, sy, sw, sh, dx, dy);
+        XFreeGC (dpy, gc);
+        XFlush (dpy);
+    }
+    void destroyBackingStore () {
+        if (backing_store)
+            XFreePixmap (display (), backing_store);
+        backing_store = 0;
+    }
+    ViewArea *m_view_area;
+    Display *dpy;
+    Drawable backing_store;
+};
+}
+
+//-----------------------------------------------------------------------------
+
 KDE_NO_CDTOR_EXPORT ViewArea::ViewArea ()
- : aspect (0.0),
+ : d (new ViewAreaPrivate (this)),
+   aspect (0.0),
    widget (0L),
    video (0L),
    surface (new ViewSurface (this)),
-   m_repaint_timer (0),
-   m_fullscreen (false) {
+   m_fullscreen (false),
+   m_event_filter_on (false) {
 }
 
 KDE_NO_CDTOR_EXPORT ViewArea::~ViewArea () {
-    if (m_repaint_timer)
-        g_source_remove (m_repaint_timer);
+     delete d;
 }
 
 KDE_NO_EXPORT void ViewArea::fullScreen () {
@@ -1372,6 +1561,7 @@ KDE_NO_EXPORT void ViewArea::fullScreen () {
     if (surface->surface) {
         cairo_surface_destroy (surface->surface);
         surface->surface = 0L;
+        d->destroyBackingStore ();
     }
 }
 
@@ -1379,6 +1569,8 @@ KDE_NO_EXPORT void ViewArea::mousePressEvent (int x, int y) {
     if (surface->node) {
         MouseVisitor visitor (event_pointer_clicked, x, y);
         surface->node->accept (&visitor);
+        if (surface->node)
+            surface->node->document ()->updateTimeout ();
     }
 }
 
@@ -1399,21 +1591,22 @@ KDE_NO_EXPORT void ViewArea::syncVisual (const IRect & rect) {
     int ew = rect.w + 2;
     int eh = rect.h + 2;
     if (!surface->surface)
-        surface->surface = cairoCreateSurface (widget, width (), height ());
-    CairoPaintVisitor visitor (surface->surface,
-            Matrix (surface->bounds.x(), surface->bounds.y(), 1.0, 1.0),
-            IRect (ex, ey, ew, eh), true);
-    surface->node->accept (&visitor);
-    if (m_repaint_timer) {
-        g_source_remove (m_repaint_timer);
-        m_repaint_timer = 0;
+        surface->surface = d->createSurface (width (), height ());
+    {
+        CairoPaintVisitor visitor (surface->surface,
+                Matrix (surface->bounds.x(), surface->bounds.y(), 1.0, 1.0),
+                IRect (ex, ey, ew, eh), true);
+        surface->node->accept (&visitor);
     }
-    //XFlush (qt_xdisplay ());
+    cairo_surface_flush (surface->surface);
+    d->swapBuffer (ex, ey, ew, eh, ex, ey);
 }
 
 KDE_NO_EXPORT void ViewArea::paintEvent (const Rect & rect) {
-    if (surface->node)
+    if (surface->node) {
         scheduleRepaint (IRect (rect.x (), rect.y (), rect.width (), rect.height ()));
+        updateScreen ();
+    }
 }
 
 KDE_NO_EXPORT void ViewArea::updateSurfaceBounds () {
@@ -1455,6 +1648,7 @@ KDE_NO_EXPORT void ViewArea::resizeEvent () {
         surface->node = n;
     }
     updateSurfaceBounds ();
+    d->resizeSurface (surface.ptr ());
     if (video_node && Mrl::SingleMode == video_node->mrl ()->view_mode)
         setAudioVideoGeometry (IRect (x, y, w, h), 0L);
 }
@@ -1523,23 +1717,25 @@ KDE_NO_EXPORT Surface *ViewArea::getSurface (Mrl *mrl) {
     if (mrl) {
         updateSurfaceBounds ();
         return surface.ptr ();
+    } else if (surface->surface) {
+        cairo_surface_destroy (surface->surface);
+        surface->surface = 0L;
+        d->destroyBackingStore ();
     }
     scheduleRepaint (IRect (0, 0, width (), height ()));
     return 0L;
 }
 
 KDE_NO_EXPORT void ViewArea::scheduleRepaint (const IRect &rect) {
-    if (m_repaint_timer)
-        m_repaint_rect = m_repaint_rect.unite (rect);
-    else {
+    if (m_repaint_rect.isEmpty ())
         m_repaint_rect = rect;
-        m_repaint_timer = g_timeout_add (20, cb_scheduledPaint, this);
-    }
+    else
+        m_repaint_rect = m_repaint_rect.unite (rect);
 }
 
-KDE_NO_EXPORT void ViewArea::timeoutPaintEvent () {
-    m_repaint_timer = 0;
+KDE_NO_EXPORT void ViewArea::updateScreen () {
     syncVisual (m_repaint_rect);
+    m_repaint_rect = IRect ();
 }
 
 KDE_NO_EXPORT int ViewArea::width () const {
@@ -1554,24 +1750,89 @@ KDE_NO_EXPORT int ViewArea::height () const {
     return h;
 }
 
+static void setXSelectInput (Display *dpy, Window wid, long mask) {
+    XWindowAttributes attr;
+    XGetWindowAttributes (dpy, wid, &attr);
+    long m = mask | attr.your_event_mask;
+    XSelectInput (dpy, wid, m);
+
+    Window r, p, *c;
+    unsigned int nr;
+    if (XQueryTree (dpy, wid, &r, &p, &c, &nr)) {
+        for (int i = 0; i < nr; ++i)
+            setXSelectInput (dpy, c[i], mask);
+        XFree (c);
+    }
+}
+
+static bool isChildWindow (Display *dpy, Window p, Window c) {
+    Window root = 0;
+    Window parent = 0;
+    Window *children;
+    unsigned int nr;
+    /*Gives 'BadWindow (invalid Window parameter)' when npp stops'
+    while (p != c && XQueryTree (dpy, c, &root, &parent, &children, &nr)) {
+        if (nr)
+            XFree (children);
+        if (parent == p || parent == root || c == root || !parent)
+            break;
+        c = parent;
+    }
+    return parent == p;*/
+    bool found = false;
+    if (XQueryTree (dpy, p, &root, &parent, &children, &nr)) {
+        for (int i = 0; i < nr; ++i) {
+            found = children[i] == c || isChildWindow (dpy, children[i], c);
+            if (found)
+                break;
+        }
+        if (nr)
+            XFree (children);
+    }
+    return found;
+}
+
 GdkFilterReturn videoWidgetFilter (GdkXEvent *xe, GdkEvent *e, gpointer d) {
     XEvent *event = (XEvent *) xe;
-        switch (event->xany.type) {
-            case KeyPress:
-            case KeyRelease:
-                if (event->xany.window == GDK_DRAWABLE_XID (GDK_DRAWABLE (
-                               VIEWAREA(((Application*) d))->videoWindow ()))) {
-                    XKeyEvent keyevent = event->xkey;
-                    keyevent.window =
-                        GDK_DRAWABLE_XID (GDK_DRAWABLE (VIEWAREA(((Application*) d))->viewWidget ()->window));
-                    XSendEvent (event->xany.display, keyevent.window, true,
-                            KeyPressMask, (XEvent *) &keyevent);
-                    return GDK_FILTER_REMOVE;
-                }
+    switch (event->xany.type) {
+        case KeyPress:
+        case KeyRelease: {
+            Window v = GDK_DRAWABLE_XID (GDK_DRAWABLE (
+                        ((ViewArea *)d)->videoWindow ()));
+            if (event->xany.window == v ||
+                    isChildWindow(event->xany.display, v, event->xkey.window)) {
+                XKeyEvent keyevent = event->xkey;
+                keyevent.window =
+                    GDK_DRAWABLE_XID (GDK_DRAWABLE (((ViewArea *)d)->viewWidget ()->window));
+                XSendEvent (event->xany.display, keyevent.window, true,
+                        KeyPressMask, (XEvent *) &keyevent);
+                return GDK_FILTER_REMOVE;
+            }
+            break;
         }
+        case MapNotify: {
+            Window v = GDK_DRAWABLE_XID (GDK_DRAWABLE (
+                        ((ViewArea *)d)->videoWindow ()));
+            if (!event->xmap.override_redirect &&
+                 (event->xany.window == v ||
+                  isChildWindow (event->xany.display, v, event->xmap.window))) {
+                setXSelectInput (event->xany.display,
+                        event->xmap.window,
+                        KeyPressMask|ExposureMask|SubstructureNotifyMask);
+            }
+        }
+    }
     return GDK_FILTER_CONTINUE;
 }
 
+KDE_NO_EXPORT void ViewArea::setEventFiltering (bool on) {
+    if (on && !m_event_filter_on)
+        gdk_window_add_filter (NULL, videoWidgetFilter, this);
+    else if (!on && m_event_filter_on)
+        gdk_window_remove_filter (NULL, videoWidgetFilter, this);
+    m_event_filter_on = on;
+}
+
 KDE_NO_EXPORT void ViewArea::setOutputWidget (Application *app, GtkWidget * w) {
     widget = w;
     gdk_window_clear (w->window);
@@ -1589,12 +1850,12 @@ KDE_NO_EXPORT void ViewArea::setOutputWidget (Application *app, GtkWidget * w) {
         attributes.wclass = GDK_INPUT_OUTPUT;
         attributes.visual = gtk_widget_get_visual (w);
         attributes.colormap = gtk_widget_get_colormap (w);
-        attributes.event_mask = gtk_widget_get_events (w);
-        attributes.event_mask &= ~(GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
+        attributes.event_mask = gtk_widget_get_events (w) |
+            GDK_SUBSTRUCTURE_MASK | GDK_EXPOSURE_MASK | GDK_KEY_PRESS_MASK;
+        attributes.event_mask &= ~GDK_BUTTON_PRESS_MASK;
         //    ~(GDK_KEY_PRESS_MASK | GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
         gint mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
         video = gdk_window_new (w->window, &attributes, mask);
-        gdk_window_add_filter (video, videoWidgetFilter, app);
         gdk_window_show (video);
         setAudioVideoGeometry (IRect (-100, -100, 50, 50), 0L);
 
@@ -1639,11 +1900,6 @@ void cbViewAreaFullscreenUnmapped (GtkWidget *w, Application *a) {
     cb_view_area_configure (w, 0L, a);
 }
 
-gboolean cb_scheduledPaint (void * vp) {
-    static_cast <ViewArea *> (vp)->timeoutPaintEvent ();
-    return false;
-}
-
 
 //#ifdef gtk < 2.8
 
