@@ -42,6 +42,7 @@ public:
     Surface *getSurface (Mrl *mrl);
     void setAudioVideoGeometry (const IRect &rect, unsigned int * bg);
     void setAudioVideoNode (NodePtr n);
+    void setEventFiltering (bool on);
     void scheduleRepaint (const IRect &rect);
     void resizeEvent ();
     void fullScreen ();
@@ -51,6 +52,8 @@ public:
     void mouseDoubleClickEvent ();
     void paintEvent (const Rect &);
     void timeoutPaintEvent ();
+    bool needsUpdateScreen () const { return !m_repaint_rect.isEmpty (); }
+    void updateScreen ();
     KDE_NO_EXPORT Rect videoGeometry () const { return m_av_geometry; }
     int width () const;
     int height () const;
@@ -69,8 +72,8 @@ private:
     GdkWindow * video;
     SurfacePtr surface;
     NodePtrW video_node;
-    int m_repaint_timer;
     bool m_fullscreen;
+    bool m_event_filter_on;
 };
 
 } // namespace KMPlayer
